"""
A low level example:
This is how JenkinsAPI creates views
"""

import json
import requests

url = "http://localhost:8080/createView"

str_view_name = "blahblah123"
params = {}  # {'name': str_view_name}
headers = {"Content-Type": "application/x-www-form-urlencoded"}
data = {
    "name": str_view_name,
    "mode": "hudson.model.ListView",
    "Submit": "OK",
    "json": json.dumps(
        {"name": str_view_name, "mode": "hudson.model.ListView"}
    ),
}
# Try 1
result = requests.post(url, params=params, data=data, headers=headers)
print(result.text.encode("UTF-8"))



"""
This example shows how to create credentials
"""

import logging
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.credential import UsernamePasswordCredential, SSHKeyCredential

log_level = getattr(logging, "DEBUG")
logging.basicConfig(level=log_level)
logger = logging.getLogger()

jenkins_url = "http://localhost:8080/"

jenkins = Jenkins(jenkins_url)

# Get a list of all global credentials
creds = jenkins.credentials
logging.info(jenkins.credentials.keys())

# Create username and password credential
creds_description1 = "My_username_credential"
cred_dict = {
    "description": creds_description1,
    "userName": "userName",
    "password": "password",
}
creds[creds_description1] = UsernamePasswordCredential(cred_dict)


# Create ssh key credential that uses private key as a value
# In jenkins credential dialog you need to paste credential
# In your code it is adviced to read it from file
# For simplicity of this example reading key from file is not shown here
def get_private_key_from_file():
    return "-----BEGIN RSA PRIVATE KEY-----"


my_private_key = get_private_key_from_file()

creds_description2 = "My_ssh_cred1"
cred_dict = {
    "description": creds_description2,
    "userName": "userName",
    "passphrase": "",
    "private_key": my_private_key,
}
creds[creds_description2] = SSHKeyCredential(cred_dict)

# Create ssh key credential that uses private key from path on Jenkins server
my_private_key = "/home/jenkins/.ssh/special_key"

creds_description3 = "My_ssh_cred2"
cred_dict = {
    "description": creds_description3,
    "userName": "userName",
    "passphrase": "",
    "private_key": my_private_key,
}
creds[creds_description3] = SSHKeyCredential(cred_dict)

# Remove credentials
# We use credential description to find specific credential. This is the only
# way to get specific credential from Jenkins via REST API
del creds[creds_description1]
del creds[creds_description2]
del creds[creds_description3]

# Remove all credentials
for cred_descr in creds.keys():
    del creds[cred_descr]



"""
How to create nested views using NestedViews Jenkins plugin

This example requires NestedViews plugin to be installed in Jenkins
You need to have at least one job in your Jenkins to see views
"""

import logging
from pkg_resources import resource_string
from jenkinsapi.views import Views
from jenkinsapi.jenkins import Jenkins

log_level = getattr(logging, "DEBUG")
logging.basicConfig(level=log_level)
logger = logging.getLogger()

jenkins_url = "http://127.0.0.1:8080/"
jenkins = Jenkins(jenkins_url)

job_name = "foo_job2"
xml = resource_string("examples", "addjob.xml")
j = jenkins.create_job(jobname=job_name, xml=xml)

# Create ListView in main view
logger.info("Attempting to create new nested view")
top_view = jenkins.views.create("TopView", Views.NESTED_VIEW)
logger.info("top_view is %s", top_view)
if top_view is None:
    logger.error("View was not created")
else:
    logger.info("View has been created")

print("top_view.views=", top_view.views.keys())
logger.info("Attempting to create view inside nested view")
sub_view = top_view.views.create("SubView")
if sub_view is None:
    logger.info("View was not created")
else:
    logger.error("View has been created")

logger.info("Attempting to delete sub_view")
del top_view.views["SubView"]
if "SubView" in top_view.views:
    logger.error("SubView was not deleted")
else:
    logger.info("SubView has been deleted")

# Another way of creating sub view
# This way sub view will have jobs in it
logger.info("Attempting to create view with jobs inside nested view")
top_view.views["SubView"] = job_name
if "SubView" not in top_view.views:
    logger.error("View was not created")
else:
    logger.info("View has been created")

logger.info("Attempting to delete sub_view")
del top_view.views["SubView"]
if "SubView" in top_view.views:
    logger.error("SubView was not deleted")
else:
    logger.info("SubView has been deleted")

logger.info("Attempting to delete top view")
del jenkins.views["TopView"]
if "TopView" not in jenkins.views:
    logger.info("View has been deleted")
else:
    logger.error("View was not deleted")

# Delete job that we created
jenkins.delete_job(job_name)



"""
System tests for `jenkinsapi.jenkins` module.
"""

import time
import logging
import pytest
from jenkinsapi.custom_exceptions import NoBuildData

log = logging.getLogger(__name__)

JOB_CONFIGS = {
    "A": """<?xml version='1.0' encoding='UTF-8'?>
<project>
  <actions/>
  <description></description>
  <keepDependencies>false</keepDependencies>
  <properties/>
  <scm class="hudson.scm.NullSCM"/>
  <canRoam>true</canRoam>
  <disabled>false</disabled>
  <blockBuildWhenDownstreamBuilding>false</blockBuildWhenDownstreamBuilding>
  <blockBuildWhenUpstreamBuilding>false</blockBuildWhenUpstreamBuilding>
  <triggers class="vector"/>
  <concurrentBuild>false</concurrentBuild>
  <builders/>
  <publishers>
    <hudson.tasks.BuildTrigger>
      <childProjects>B</childProjects>
      <threshold>
        <name>SUCCESS</name>
        <ordinal>0</ordinal>
        <color>BLUE</color>
      </threshold>
    </hudson.tasks.BuildTrigger>
  </publishers>
  <buildWrappers/>
</project>""",
    "B": """<?xml version='1.0' encoding='UTF-8'?>
<project>
  <actions/>
  <description></description>
  <keepDependencies>false</keepDependencies>
  <properties/>
  <scm class="hudson.scm.NullSCM"/>
  <canRoam>true</canRoam>
  <disabled>false</disabled>
  <blockBuildWhenDownstreamBuilding>false</blockBuildWhenDownstreamBuilding>
  <blockBuildWhenUpstreamBuilding>false</blockBuildWhenUpstreamBuilding>
  <triggers class="vector"/>
  <concurrentBuild>false</concurrentBuild>
  <builders/>
  <publishers>
    <hudson.tasks.BuildTrigger>
      <childProjects>C</childProjects>
      <threshold>
        <name>SUCCESS</name>
        <ordinal>0</ordinal>
        <color>BLUE</color>
      </threshold>
    </hudson.tasks.BuildTrigger>
  </publishers>
  <buildWrappers/>
</project>""",
    "C": """<?xml version='1.0' encoding='UTF-8'?>
<project>
  <actions/>
  <description></description>
  <keepDependencies>false</keepDependencies>
  <properties/>
  <scm class="hudson.scm.NullSCM"/>
  <canRoam>true</canRoam>
  <disabled>false</disabled>
  <blockBuildWhenDownstreamBuilding>false</blockBuildWhenDownstreamBuilding>
  <blockBuildWhenUpstreamBuilding>false</blockBuildWhenUpstreamBuilding>
  <triggers class="vector"/>
  <concurrentBuild>false</concurrentBuild>
  <builders/>
  <publishers/>
  <buildWrappers/>
</project>""",
}

DELAY = 10


def test_stream_relationship(jenkins):
    """
    Can we keep track of the relationships between upstream & downstream jobs?
    """
    for job_name, job_config in JOB_CONFIGS.items():
        jenkins.create_job(job_name, job_config)

    time.sleep(1)

    jenkins["A"].invoke()

    for _ in range(10):
        try:
            jenkins["C"].get_last_completed_buildnumber() > 0
        except NoBuildData:
            log.info(
                "Waiting %i seconds for until the final job has run", DELAY
            )
            time.sleep(DELAY)
        else:
            break
    else:
        pytest.fail("Jenkins took too long to run these jobs")

    assert jenkins["C"].get_upstream_jobs() == [jenkins["B"]]
    assert jenkins["B"].get_upstream_jobs() == [jenkins["A"]]

    assert jenkins["A"].get_downstream_jobs() == [jenkins["B"]]
    assert jenkins["B"].get_downstream_jobs() == [jenkins["C"]]



"""
How to create views
"""

import logging
from pkg_resources import resource_string
from jenkinsapi.jenkins import Jenkins

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

jenkins_url = "http://localhost:8080/"

jenkins = Jenkins(jenkins_url, lazy=True)

# Create ListView in main view
logger.info("Attempting to create new view")
test_view_name = "SimpleListView"

# Views object appears as a dictionary of views
if test_view_name not in jenkins.views:
    new_view = jenkins.views.create(test_view_name)
    if new_view is None:
        logger.error("View %s was not created", test_view_name)
    else:
        logger.info(
            "View %s has been created: %s", new_view.name, new_view.baseurl
        )
else:
    logger.info("View %s already exists", test_view_name)

# No error is raised if view already exists
logger.info("Attempting to create view that already exists")
my_view = jenkins.views.create(test_view_name)

logger.info("Create job and assign it to a view")
job_name = "foo_job2"
xml = resource_string("examples", "addjob.xml")

my_job = jenkins.create_job(jobname=job_name, xml=xml)

# add_job supports two parameters: job_name and job object
# passing job object will remove verification calls to Jenkins
my_view.add_job(job_name, my_job)
assert len(my_view) == 1

logger.info("Attempting to delete view that already exists")
del jenkins.views[test_view_name]

if test_view_name in jenkins.views:
    logger.error("View was not deleted")
else:
    logger.info("View has been deleted")

# No error will be raised when attempting to remove non-existing view
logger.info("Attempting to delete view that does not exist")
del jenkins.views[test_view_name]

# Create CategorizedJobsView
config = """
<org.jenkinsci.plugins.categorizedview.CategorizedJobsView>
  <categorizationCriteria>
    <org.jenkinsci.plugins.categorizedview.GroupingRule>
      <groupRegex>.dev.</groupRegex>
      <namingRule>Development</namingRule>
    </org.jenkinsci.plugins.categorizedview.GroupingRule>
    <org.jenkinsci.plugins.categorizedview.GroupingRule>
      <groupRegex>.hml.</groupRegex>
      <namingRule>Homologation</namingRule>
    </org.jenkinsci.plugins.categorizedview.GroupingRule>
  </categorizationCriteria>
</org.jenkinsci.plugins.categorizedview.CategorizedJobsView>
"""
view = jenkins.views.create(
    "My categorized jobs view", jenkins.views.CATEGORIZED_VIEW, config=config
)



import os
import logging


logging.basicConfig(
    format="%(module)s.%(funcName)s %(levelname)s: %(message)s",
    level=logging.INFO,
)

level = (
    logging.WARNING
    if "LOG_LEVEL" not in os.environ
    else os.environ["LOG_LEVEL"].upper().strip()
)

modules = [
    "requests.packages.urllib3.connectionpool",
    "requests",
    "urllib3",
    "urllib3.connectionpool",
]

for module_name in modules:
    logger = logging.getLogger(module_name)
    logger.setLevel(level)



"""
jenkinsapi class for invoking Jenkins
"""

import os
import sys
import logging
import optparse
from jenkinsapi import jenkins

log = logging.getLogger(__name__)


class JenkinsInvoke(object):
    """
    JenkinsInvoke object implements class to call from command line
    """

    @classmethod
    def mkparser(cls):
        parser = optparse.OptionParser()
        DEFAULT_BASEURL = os.environ.get(
            "JENKINS_URL", "http://localhost/jenkins"
        )
        parser.help_text = (
            "Execute a number of jenkins jobs on the server of your choice."
            + " Optionally block until the jobs are complete."
        )
        parser.add_option(
            "-J",
            "--jenkinsbase",
            dest="baseurl",
            help="Base URL for the Jenkins server, default is %s"
            % DEFAULT_BASEURL,
            type="str",
            default=DEFAULT_BASEURL,
        )
        parser.add_option(
            "--username",
            "-u",
            dest="username",
            help="Username for jenkins authentification",
            type="str",
            default=None,
        )
        parser.add_option(
            "--password",
            "-p",
            dest="password",
            help="password for jenkins user auth",
            type="str",
            default=None,
        )
        parser.add_option(
            "-b",
            "--block",
            dest="block",
            action="store_true",
            default=False,
            help="Block until each of the jobs is complete.",
        )
        parser.add_option(
            "-t",
            "--token",
            dest="token",
            help="Optional security token.",
            default=None,
        )
        return parser

    @classmethod
    def main(cls):
        parser = cls.mkparser()
        options, args = parser.parse_args()
        try:
            assert args, "Need to specify at least one job name"
        except AssertionError as err:
            log.critical(err.message)
            parser.print_help()
            sys.exit(1)
        invoker = cls(options, args)
        invoker()

    def __init__(self, options, jobs):
        self.options = options
        self.jobs = jobs
        self.api = self._get_api(
            baseurl=options.baseurl,
            username=options.username,
            password=options.password,
        )

    def _get_api(self, baseurl, username, password):
        return jenkins.Jenkins(baseurl, username, password)

    def __call__(self):
        for job in self.jobs:
            self.invokejob(
                job, block=self.options.block, token=self.options.token
            )

    def invokejob(self, jobname, block, token):
        assert isinstance(block, bool)
        assert isinstance(jobname, str)
        assert token is None or isinstance(token, str)
        job = self.api.get_job(jobname)
        job.invoke(securitytoken=token, block=block)


def main():
    logging.basicConfig()
    logging.getLogger("").setLevel(logging.INFO)
    JenkinsInvoke.main()



"""
System tests for restarting jenkins

NB: this test will be very time consuming because
    after restart it will wait for jenkins to boot
"""

import time
import logging
import pytest
from requests import HTTPError, ConnectionError

log = logging.getLogger(__name__)


def wait_for_restart(jenkins):
    wait = 15
    count = 0
    max_count = 30
    success = False
    msg = (
        "Jenkins has not restarted yet! (This is try %s of %s, "
        "waited %s seconds so far) "
        "Sleeping %s seconds and trying again..."
    )

    while count < max_count or not success:
        time.sleep(wait)
        try:
            jenkins.poll()
            log.info("Jenkins restarted successfully.")
            success = True
            break
        except HTTPError as ex:
            log.info(ex)
        except ConnectionError as ex:
            log.info(ex)

        log.info(msg, count + 1, max_count, count * wait, wait)
        count += 1

    if not success:
        msg = (
            "Jenkins did not come back from safe restart! "
            "Waited {0} seconds altogether.  This "
            "failure may cause other failures."
        )
        log.critical(msg.format(count * wait))
        pytest.fail(msg)


def test_safe_restart_wait(jenkins):
    jenkins.poll()  # jenkins should be alive
    jenkins.safe_restart()  # restart and wait for reboot (default)
    jenkins.poll()  # jenkins should be alive again


def test_safe_restart_dont_wait(jenkins):
    jenkins.poll()  # jenkins should be alive
    jenkins.safe_restart(wait_for_reboot=False)
    # Jenkins sleeps for 10 seconds before actually restarting
    time.sleep(11)
    with pytest.raises((HTTPError, ConnectionError)):
        # this is a 503: jenkins is still restarting
        jenkins.poll()
    # the test is now complete, but other tests cannot run until
    # jenkins has finished restarted.  to avoid cascading failure
    # we have to wait for reboot to finish.
    wait_for_restart(jenkins)



import io
import time
import json
import logging
import pytest

from urllib.parse import urljoin
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.utils.crumb_requester import CrumbRequester
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import JOB_WITH_FILE


log = logging.getLogger(__name__)

DEFAULT_JENKINS_PORT = 8080

ENABLE_CRUMBS_CONFIG = {
    "hudson-security-csrf-GlobalCrumbIssuerConfiguration": {
        "csrf": {
            "issuer": {
                "value": "0",
                "stapler-class": "hudson.security.csrf.DefaultCrumbIssuer",
                "$class": "hudson.security.csrf.DefaultCrumbIssuer",
                "excludeClientIPFromCrumb": False,
            }
        }
    }
}

DISABLE_CRUMBS_CONFIG = {
    "hudson-security-csrf-GlobalCrumbIssuerConfiguration": {},
}

SECURITY_SETTINGS = {
    "": "0",
    "markupFormatter": {
        "stapler-class": "hudson.markup.EscapedMarkupFormatter",
        "$class": "hudson.markup.EscapedMarkupFormatter",
    },
    "org-jenkinsci-main-modules-sshd-SSHD": {
        "port": {"value": "", "type": "disabled"}
    },
    "jenkins-CLI": {"enabled": False},
    # This is not required if envinject plugin is not installed
    # but since it is installed for test suite - we must have this config
    # If this is not present - Jenkins will return error
    "org-jenkinsci-plugins-envinject-EnvInjectPluginConfiguration": {
        "enablePermissions": False,
        "hideInjectedVars": False,
        "enableLoadingFromMaster": False,
    },
    "jenkins-model-DownloadSettings": {"useBrowser": False},
    "slaveAgentPort": {"value": "", "type": "disable"},
    "agentProtocol": [
        "CLI-connect",
        "CLI2-connect",
        "JNLP-connect",
        "JNLP2-connect",
        "JNLP4-connect",
    ],
    "core:apply": "",
}


@pytest.fixture(scope="function")
def crumbed_jenkins(jenkins):
    ENABLE_CRUMBS_CONFIG.update(SECURITY_SETTINGS)
    DISABLE_CRUMBS_CONFIG.update(SECURITY_SETTINGS)

    jenkins.requester.post_and_confirm_status(
        urljoin(jenkins.baseurl, "/configureSecurity/configure"),
        data={"Submit": "save", "json": json.dumps(ENABLE_CRUMBS_CONFIG)},
        headers={"Content-Type": "application/x-www-form-urlencoded"},
    )
    log.info("Enabled Jenkins security")

    crumbed = Jenkins(
        jenkins.baseurl, requester=CrumbRequester(baseurl=jenkins.baseurl)
    )

    yield crumbed

    crumbed.requester.post_and_confirm_status(
        jenkins.baseurl + "/configureSecurity/configure",
        data={"Submit": "save", "json": json.dumps(DISABLE_CRUMBS_CONFIG)},
        headers={"Content-Type": "application/x-www-form-urlencoded"},
    )
    log.info("Disabled Jenkins security")


def test_invoke_job_with_file(crumbed_jenkins):
    file_data = random_string()
    param_file = io.BytesIO(file_data.encode("utf-8"))

    job_name = "create1_%s" % random_string()
    job = crumbed_jenkins.create_job(job_name, JOB_WITH_FILE)

    assert job.has_params()
    assert len(job.get_params_list())

    job.invoke(block=True, files={"file.txt": param_file})

    build = job.get_last_build()
    while build.is_running():
        time.sleep(0.25)

    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True
    art_file = artifacts["file.txt"]
    assert art_file.get_data().decode("utf-8").strip() == file_data



# -*- coding: utf-8 -*-
import pytest
import mock
import json
from . import configs
from jenkinsapi.job import Job
from jenkinsapi.build import Build
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import NoBuildData


@pytest.fixture(scope="function")
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)
    new_jenkins = Jenkins("http://halob:8080/")

    return new_jenkins


@pytest.fixture(scope="function")
def job(jenkins, monkeypatch):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return configs.JOB_DATA

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)

    new_job = Job("http://halob:8080/job/foo/", "foo", jenkins)

    return new_job


@pytest.fixture(scope="function")
def job_tree(jenkins, monkeypatch):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        if tree is not None and "builds" in tree:
            return {"builds": configs.JOB_DATA["builds"]}
        else:
            return {"lastBuild": configs.JOB_DATA["lastBuild"]}

    monkeypatch.setattr(Job, "get_data", fake_get_data)

    new_job = Job("http://halob:8080/job/foo/", "foo", jenkins)

    return new_job


@pytest.fixture(scope="function")
def job_tree_empty(jenkins, monkeypatch):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Job, "get_data", fake_get_data)

    new_job = Job("http://halob:8080/job/foo/", "foo", jenkins)

    return new_job


def test_repr(job):
    # Can we produce a repr string for this object
    assert repr(job) == "<jenkinsapi.job.Job foo>"


def test_name(job):
    with pytest.raises(AttributeError):
        job.id()
    assert job.name == "foo"


def test_next_build_number(job):
    assert job.get_next_build_number() == 4


def test_lastcompleted_build_number(job):
    assert job.get_last_completed_buildnumber() == 3


def test_lastgood_build_number(job):
    assert job.get_last_good_buildnumber() == 3


def test_special_urls(job):
    assert job.baseurl == "http://halob:8080/job/foo"
    assert job.get_delete_url() == "http://halob:8080/job/foo/doDelete"
    assert job.get_rename_url() == "http://halob:8080/job/foo/doRename"


def test_get_description(job):
    assert job.get_description() == "test job"


def test_get_build_triggerurl(job):
    assert (
        job.get_build_triggerurl()
        == "http://halob:8080/job/foo/buildWithParameters"
    )


def test_wrong__mk_json_from_build_parameters(job):
    with pytest.raises(ValueError) as ar:
        job._mk_json_from_build_parameters(build_params="bad parameter")

    assert str(ar.value) == "Build parameters must be a dict"


def test_unicode_mk_json(job):
    json = job._mk_json_from_build_parameters(
        {"age": 20, "name": "品品", "country": "USA", "height": 1.88}
    )
    assert isinstance(json, dict)


def test_wrong_field__build_id_for_type(job):
    with pytest.raises(AssertionError):
        job._buildid_for_type("wrong")


def test_get_last_good_buildnumber(job):
    ret = job.get_last_good_buildnumber()
    assert ret == 3


def test_get_last_stable_buildnumber(job):
    ret = job.get_last_stable_buildnumber()
    assert ret == 3


def test_get_last_failed_buildnumber(job):
    with pytest.raises(NoBuildData):
        job.get_last_failed_buildnumber()


def test_get_last_buildnumber(job):
    ret = job.get_last_buildnumber()
    assert ret == 4


def test_get_last_completed_buildnumber(job):
    ret = job.get_last_completed_buildnumber()
    assert ret == 3


def test_get_build_dict(job_tree):
    ret = job_tree.get_build_dict()
    assert isinstance(ret, dict)
    assert len(ret) == 4


def test_get_build_metadata(job_tree):
    with pytest.raises(ValueError) as ve:
        job_tree.get_build_metadata("abc")

    assert 'Parameter "buildNumber" must be int' in str(ve.value)


def test_nobuilds_get_build_dict(job_tree_empty):
    with pytest.raises(NoBuildData):
        job_tree_empty.get_build_dict()


def test_get_build_ids(job):
    # We don't want to deal with listreverseiterator here
    # So we convert result to a list
    ret = list(job.get_build_ids())
    assert isinstance(ret, list)
    assert len(ret) == 4


def test_nobuilds_get_revision_dict(jenkins, monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {"name": "foo"}

    monkeypatch.setattr(Job, "_poll", fake_poll)
    job = Job("http://halob:8080/job/foo/", "foo", jenkins)
    with pytest.raises(NoBuildData):
        job.get_revision_dict()


def test_nobuilds_get_last_build(jenkins, monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {"name": "foo"}

    monkeypatch.setattr(Job, "_poll", fake_poll)

    job = Job("http://halob:8080/job/foo/", "foo", jenkins)
    with pytest.raises(NoBuildData):
        job.get_last_build()


def test__add_missing_builds_not_all_loaded(jenkins, monkeypatch):
    def fake_get_data(cls, url, tree):  # pylint: disable=unused-argument
        return configs.JOB_DATA.copy()

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)
    job = Job("http://halob:8080/job/foo/", "foo", jenkins)

    # to test this function we change data to not have one build
    # and set it to mark that firstBuild was not loaded
    # in that condition function will call j.get_data
    # and will use syntetic field 'allBuilds' to
    # repopulate 'builds' field with all builds
    mock_data = configs.JOB_DATA.copy()
    mock_data["firstBuild"] = {"number": 1}
    del mock_data["builds"][-1]
    job._data = mock_data

    assert len(mock_data["builds"]) == 2
    new_data = job._add_missing_builds(mock_data)
    assert len(new_data["builds"]) == 3


def test__add_missing_builds_no_first_build(job, mocker):
    mocker.spy(JenkinsBase, "get_data")

    initial_call_count = job.get_data.call_count
    mock_data = configs.JOB_DATA.copy()
    mock_data["firstBuild"] = None
    job._data = mock_data

    job._add_missing_builds(mock_data)

    assert initial_call_count == job.get_data.call_count


@mock.patch.object(JenkinsBase, "get_data")
def test__add_missing_builds_no_builds(job, mocker):
    mocker.spy(JenkinsBase, "get_data")

    initial_call_count = job.get_data.call_count
    mock_data = configs.JOB_DATA.copy()
    mock_data["builds"] = None
    job._data = mock_data

    job._add_missing_builds(mock_data)

    assert initial_call_count == job.get_data.call_count


def test_get_params(job):
    params = list(job.get_params())
    assert len(params) == 2


def test_get_params_list(job):
    assert job.has_params() is True
    params = job.get_params_list()

    assert isinstance(params, list)
    assert len(params) == 2
    assert params == ["param1", "param2"]


def json_equal(json_a, json_b):
    dict_a = json.loads(json_a)
    dict_b = json.loads(json_b)
    assert dict_a == dict_b


def test_get_json_for_single_param():
    params = {"B": "one two three"}
    expected = (
        '{"parameter": {"name": "B", "value": "one two three"}, '
        '"statusCode": "303", "redirectTo": "."}'
    )
    json_equal(Job.mk_json_from_build_parameters(params), expected)


def test_get_json_for_many_params():
    params = {"B": "Honey", "A": "Boo", "C": 2}
    expected = (
        '{"parameter": [{"name": "A", "value": "Boo"}, '
        '{"name": "B", "value": "Honey"}, '
        '{"name": "C", "value": 2}], '
        '"statusCode": "303", "redirectTo": "."}'
    )

    json_equal(Job.mk_json_from_build_parameters(params), expected)


def test__mk_json_from_build_parameters(job):
    params = {"param1": "value1", "param2": "value2"}
    expected = {
        "parameter": [
            {"name": "param1", "value": "value1"},
            {"name": "param2", "value": "value2"},
        ]
    }
    result = job._mk_json_from_build_parameters(build_params=params)
    assert isinstance(result, dict)

    assert result == expected


def test_wrong_mk_json_from_build_parameters(job):
    with pytest.raises(ValueError) as ar:
        job.mk_json_from_build_parameters(build_params="bad parameter")

    assert "Build parameters must be a dict" in str(ar.value)


def test_get_build_by_params(jenkins, monkeypatch, mocker):
    build_params = {"param1": "value1"}
    fake_builds = (
        mocker.Mock(get_params=lambda: {}),
        mocker.Mock(get_params=lambda: {}),
        mocker.Mock(get_params=lambda: build_params),
    )

    build_call_count = [0]

    def fake_get_build(cls, number):  # pylint: disable=unused-argument
        build_call_count[0] += 1
        return fake_builds[number - 1]

    monkeypatch.setattr(Job, "get_first_buildnumber", lambda x: 1)
    monkeypatch.setattr(Job, "get_last_buildnumber", lambda x: 3)
    monkeypatch.setattr(Job, "get_build", fake_get_build)
    mocker.spy(Build, "get_params")
    mocker.spy(Job, "get_build")

    job = Job("http://localhost/jobs/foo", "foo", jenkins)

    result = job.get_build_by_params(build_params)

    assert job.get_build.call_count == 3
    assert build_call_count[0] == 3
    assert result == fake_builds[2]


def test_get_build_by_params_not_found(jenkins, monkeypatch, mocker):
    build_params = {"param1": "value1"}
    fake_builds = (
        mocker.Mock(get_params=lambda: {}),
        mocker.Mock(get_params=lambda: {}),
        mocker.Mock(get_params=lambda: {}),
    )

    build_call_count = [0]

    def fake_get_build(cls, number):  # pylint: disable=unused-argument
        build_call_count[0] += 1
        return fake_builds[number - 1]

    monkeypatch.setattr(Job, "get_first_buildnumber", lambda x: 1)
    monkeypatch.setattr(Job, "get_last_buildnumber", lambda x: 3)
    monkeypatch.setattr(Job, "get_build", fake_get_build)
    mocker.spy(Build, "get_params")
    mocker.spy(Job, "get_build")

    job = Job("http://localhost/jobs/foo", "foo", jenkins)

    with pytest.raises(NoBuildData):
        job.get_build_by_params(build_params)

    assert job.get_build.call_count == 3
    assert build_call_count[0] == 3



import pytest
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.nodes import Nodes
from jenkinsapi.node import Node


DATA0 = {
    "assignedLabels": [{}],
    "description": None,
    "jobs": [],
    "mode": "NORMAL",
    "nodeDescription": "the master Jenkins node",
    "nodeName": "",
    "numExecutors": 2,
    "overallLoad": {},
    "primaryView": {"name": "All", "url": "http://halob:8080/"},
    "quietingDown": False,
    "slaveAgentPort": 0,
    "unlabeledLoad": {},
    "useCrumbs": False,
    "useSecurity": False,
    "views": [
        {"name": "All", "url": "http://halob:8080/"},
        {"name": "FodFanFo", "url": "http://halob:8080/view/FodFanFo/"},
    ],
}

DATA1 = {
    "busyExecutors": 0,
    "computer": [
        {
            "actions": [],
            "displayName": "master",
            "executors": [{}, {}],
            "icon": "computer.png",
            "idle": True,
            "jnlpAgent": False,
            "launchSupported": True,
            "loadStatistics": {},
            "manualLaunchAllowed": True,
            "monitorData": {
                "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
                "hudson.node_monitors.ClockMonitor": {"diff": 0},
                "hudson.node_monitors.DiskSpaceMonitor": {
                    "path": "/var/lib/jenkins",
                    "size": 671924924416,
                },
                "hudson.node_monitors.ResponseTimeMonitor": {"average": 0},
                "hudson.node_monitors.SwapSpaceMonitor": {
                    "availablePhysicalMemory": 3174686720,
                    "availableSwapSpace": 17163087872,
                    "totalPhysicalMemory": 16810180608,
                    "totalSwapSpace": 17163087872,
                },
                "hudson.node_monitors.TemporarySpaceMonitor": {
                    "path": "/tmp",
                    "size": 671924924416,
                },
            },
            "numExecutors": 2,
            "offline": False,
            "offlineCause": None,
            "oneOffExecutors": [],
            "temporarilyOffline": False,
        },
        {
            "actions": [],
            "displayName": "bobnit",
            "executors": [{}],
            "icon": "computer-x.png",
            "idle": True,
            "jnlpAgent": False,
            "launchSupported": True,
            "loadStatistics": {},
            "manualLaunchAllowed": True,
            "monitorData": {
                "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
                "hudson.node_monitors.ClockMonitor": {"diff": 4261},
                "hudson.node_monitors.DiskSpaceMonitor": {
                    "path": "/home/sal/jenkins",
                    "size": 169784860672,
                },
                "hudson.node_monitors.ResponseTimeMonitor": {"average": 29},
                "hudson.node_monitors.SwapSpaceMonitor": {
                    "availablePhysicalMemory": 4570710016,
                    "availableSwapSpace": 12195983360,
                    "totalPhysicalMemory": 8374497280,
                    "totalSwapSpace": 12195983360,
                },
                "hudson.node_monitors.TemporarySpaceMonitor": {
                    "path": "/tmp",
                    "size": 249737277440,
                },
            },
            "numExecutors": 1,
            "offline": True,
            "offlineCause": {},
            "oneOffExecutors": [],
            "temporarilyOffline": False,
        },
        {
            "actions": [],
            "displayName": "halob",
            "executors": [{}],
            "icon": "computer-x.png",
            "idle": True,
            "jnlpAgent": True,
            "launchSupported": False,
            "loadStatistics": {},
            "manualLaunchAllowed": True,
            "monitorData": {
                "hudson.node_monitors.ArchitectureMonitor": None,
                "hudson.node_monitors.ClockMonitor": None,
                "hudson.node_monitors.DiskSpaceMonitor": None,
                "hudson.node_monitors.ResponseTimeMonitor": None,
                "hudson.node_monitors.SwapSpaceMonitor": None,
                "hudson.node_monitors.TemporarySpaceMonitor": None,
            },
            "numExecutors": 1,
            "offline": True,
            "offlineCause": None,
            "oneOffExecutors": [],
            "temporarilyOffline": False,
        },
    ],
    "displayName": "nodes",
    "totalExecutors": 2,
}

DATA2 = {
    "actions": [],
    "displayName": "master",
    "executors": [{}, {}],
    "icon": "computer.png",
    "idle": True,
    "jnlpAgent": False,
    "launchSupported": True,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ClockMonitor": {"diff": 0},
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/var/lib/jenkins",
            "size": 671942561792,
        },
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 0},
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 2989916160,
            "availableSwapSpace": 17163087872,
            "totalPhysicalMemory": 16810180608,
            "totalSwapSpace": 17163087872,
        },
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 671942561792,
        },
    },
    "numExecutors": 2,
    "offline": False,
    "offlineCause": None,
    "oneOffExecutors": [],
    "temporarilyOffline": False,
}

DATA3 = {
    "actions": [],
    "displayName": "halob",
    "executors": [{}],
    "icon": "computer-x.png",
    "idle": True,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.ArchitectureMonitor": None,
        "hudson.node_monitors.ClockMonitor": None,
        "hudson.node_monitors.DiskSpaceMonitor": None,
        "hudson.node_monitors.ResponseTimeMonitor": None,
        "hudson.node_monitors.SwapSpaceMonitor": None,
        "hudson.node_monitors.TemporarySpaceMonitor": None,
    },
    "numExecutors": 1,
    "offline": True,
    "offlineCause": None,
    "oneOffExecutors": [],
    "temporarilyOffline": False,
}


@pytest.fixture(scope="function")
def nodes(monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA0

    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)

    def fake_nodes_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA1

    monkeypatch.setattr(Nodes, "_poll", fake_nodes_poll)
    jenkins = Jenkins("http://foo:8080")
    return jenkins.get_nodes()


def fake_node_poll(self, tree=None):  # pylint: disable=unused-argument
    """
    Fakes a poll of data by returning the correct section of
    the DATA1 test block.
    """
    for node_poll in DATA1["computer"]:
        if node_poll["displayName"] == self.name:
            return node_poll
    return DATA2


def test_repr(nodes):
    # Can we produce a repr string for this object
    repr(nodes)


def test_baseurl(nodes):
    assert nodes.baseurl == "http://foo:8080/computer"


def test_get_master_node(nodes, monkeypatch):
    monkeypatch.setattr(Node, "_poll", fake_node_poll)

    node = nodes["master"]
    assert isinstance(node, Node)


def test_get_nonmaster_node(nodes, monkeypatch):
    monkeypatch.setattr(Node, "_poll", fake_node_poll)

    node = nodes["halob"]
    assert isinstance(node, Node)


def test_iterkeys(nodes):
    expected_names = set(["master", "bobnit", "halob"])
    actual_names = set([n for n in nodes.iterkeys()])
    assert actual_names == expected_names


def test_keys(nodes):
    expected_names = set(["master", "bobnit", "halob"])
    actual_names = set(nodes.keys())
    assert actual_names == expected_names


def items_test_case(nodes_method, monkeypatch):
    monkeypatch.setattr(Node, "_poll", fake_node_poll)

    expected_names = set(["master", "bobnit", "halob"])

    actual_names = set()
    for name, node in nodes_method():
        assert name == node.name
        assert isinstance(node, Node)

        actual_names.add(name)

    assert actual_names == expected_names


def test_iteritems(nodes, monkeypatch):
    items_test_case(nodes.iteritems, monkeypatch)


def test_items(nodes, monkeypatch):
    items_test_case(nodes.items, monkeypatch)


def values_test_case(nodes_method, monkeypatch):
    monkeypatch.setattr(Node, "_poll", fake_node_poll)

    expected_names = set(["master", "bobnit", "halob"])

    actual_names = set()
    for node in nodes_method():
        assert isinstance(node, Node)
        actual_names.add(node.name)

    assert actual_names == expected_names


def test_itervalues(nodes, monkeypatch):
    values_test_case(nodes.itervalues, monkeypatch)


def test_values(nodes, monkeypatch):
    values_test_case(nodes.values, monkeypatch)



import requests
import pytest
import pytz
from . import configs
import datetime
import warnings
from typing import List
from jenkinsapi.build import Build
from jenkinsapi.job import Job
from jenkinsapi.artifact import Artifact


@pytest.fixture(scope="function")
def jenkins(mocker):
    return mocker.MagicMock()


@pytest.fixture(scope="function")
def job(monkeypatch, jenkins) -> Job:
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return configs.JOB_DATA

    monkeypatch.setattr(Job, "_poll", fake_poll)

    fake_job: Job = Job("http://", "Fake_Job", jenkins)
    return fake_job


@pytest.fixture(scope="function")
def build(job, monkeypatch) -> Build:
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return configs.BUILD_DATA

    monkeypatch.setattr(Build, "_poll", fake_poll)

    return Build("http://", 97, job)


@pytest.fixture(scope="function")
def build_pipeline(job, monkeypatch) -> Build:
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return configs.BUILD_DATA_PIPELINE

    monkeypatch.setattr(Build, "_poll", fake_poll)

    return Build("http://", 97, job)


def test_timestamp(build) -> None:
    assert isinstance(build.get_timestamp(), datetime.datetime)

    expected: datetime.datetime = pytz.utc.localize(
        datetime.datetime(2013, 5, 31, 23, 15, 40)
    )

    assert build.get_timestamp() == expected


def test_name(build) -> None:
    with pytest.raises(AttributeError):
        build.id()
    assert build.name == "foo #1"


def test_duration(build) -> None:
    expected = datetime.timedelta(milliseconds=5782)
    assert build.get_duration() == expected
    assert build.get_duration().seconds == 5
    assert build.get_duration().microseconds == 782000
    assert str(build.get_duration()) == "0:00:05.782000"


def test_get_causes(build) -> None:
    assert build.get_causes() == [
        {
            "shortDescription": "Started by user anonymous",
            "userId": None,
            "userName": "anonymous",
            "upstreamProject": "parentBuild",
            "upstreamBuild": 1,
        }
    ]


def test_get_changeset(build):
    assert build.get_changeset_items() == [
        {
            "affectedPaths": ["content/rcm/v00-rcm-xccdf.xml"],
            "author": {
                "absoluteUrl": "http://jenkins_url/user/username79",
                "fullName": "username",
            },
            "commitId": "3097",
            "timestamp": 1414398423091,
            "date": "2014-10-27T08:27:03.091288Z",
            "msg": "commit message",
            "paths": [
                {"editType": "edit", "file": "/some/path/of/changed_file"}
            ],
            "revision": 3097,
            "user": "username",
        }
    ]


def test_get_changeset_pipeline(build_pipeline):
    assert build_pipeline.get_changeset_items() == [
        {
            "affectedPaths": ["content/rcm/v00-rcm-xccdf.xml"],
            "author": {
                "absoluteUrl": "http://jenkins_url/user/username79",
                "fullName": "username",
            },
            "commitId": "3097",
            "timestamp": 1414398423091,
            "date": "2014-10-27T08:27:03.091288Z",
            "msg": "commit message",
            "paths": [
                {"editType": "edit", "file": "/some/path/of/changed_file"}
            ],
            "revision": 3097,
            "user": "username",
        }
    ]


def test_get_description(build):
    assert build.get_description() == "Best build ever!"


def test_get_slave(build):
    assert build.get_slave() == "localhost"


def test_get_revision_no_scm(build):
    """with no scm, get_revision should return None"""
    assert build.get_revision() is None


def test_downstream(build):
    expected = ["test1", "test2"]
    assert build.get_downstream_job_names() == expected


def test_get_params(build):
    expected = {
        "first_param": "first_value",
        "second_param": "second_value",
    }
    build._data = {
        "actions": [
            {
                "_class": "hudson.model.ParametersAction",
                "parameters": [
                    {"name": "first_param", "value": "first_value"},
                    {"name": "second_param", "value": "second_value"},
                ],
            }
        ]
    }
    params = build.get_params()
    assert params == expected


def test_get_build_url(build):
    expected = "http://foo/1"
    build._data = {"url": "http://foo/1"}
    url = build.get_build_url()
    assert url == expected


def test_get_params_different_order(build):
    """
    Dictionary with `parameters` key is not always the first element in
    `actions` list, so we need to search through whole array. This test
    covers such a case
    """
    expected = {
        "first_param": "first_value",
        "second_param": "second_value",
    }
    build._data = {
        "actions": [
            {
                "not_parameters": "some_data",
            },
            {
                "another_action": "some_value",
            },
            {
                "_class": "hudson.model.ParametersAction",
                "parameters": [
                    {"name": "first_param", "value": "first_value"},
                    {"name": "second_param", "value": "second_value"},
                ],
            },
        ]
    }
    params = build.get_params()
    assert params == expected


def test_only_ParametersAction_parameters_considered(build):
    """Actions other than ParametersAction can have dicts called parameters."""
    expected = {
        "param": "value",
    }
    build._data = {
        "actions": [
            {
                "_class": "hudson.model.SomeOtherAction",
                "parameters": [
                    {"name": "Not", "value": "OurValue"},
                ],
            },
            {
                "_class": "hudson.model.ParametersAction",
                "parameters": [
                    {"name": "param", "value": "value"},
                ],
            },
        ]
    }
    params = build.get_params()
    assert params == expected


def test_ParametersWithNoValueSetValueNone_issue_583(build):
    """SecretParameters don't share their value in the API."""
    expected = {
        "some-secret": None,
    }
    build._data = {
        "actions": [
            {
                "_class": "hudson.model.ParametersAction",
                "parameters": [
                    {"name": "some-secret"},
                ],
            }
        ]
    }
    params = build.get_params()
    assert params == expected


def test_build_env_vars(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        return configs.BUILD_ENV_VARS

    monkeypatch.setattr(Build, "get_data", fake_get_data)
    assert build.get_env_vars() == configs.BUILD_ENV_VARS["envMap"]


def test_build_env_vars_wo_injected_env_vars_plugin(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        raise requests.HTTPError("404")

    monkeypatch.setattr(Build, "get_data", fake_get_data)

    with pytest.raises(requests.HTTPError) as excinfo:
        with pytest.warns(UserWarning) as record:
            build.get_env_vars()
    assert "404" == str(excinfo.value)
    assert len(record) == 1
    expected = UserWarning(
        "Make sure the Environment Injector plugin is installed."
    )
    assert str(record[0].message) == str(expected)


def test_build_env_vars_other_exception(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        raise ValueError()

    monkeypatch.setattr(Build, "get_data", fake_get_data)

    with pytest.raises(Exception) as excinfo:
        with warnings.catch_warnings():
            warnings.simplefilter("error")
            build.get_env_vars()
    assert "" == str(excinfo.value)


def test_build_get_status(build) -> None:
    assert build.get_status() == "SUCCESS"


def test_build_get_params_return_empty_dict(build) -> None:
    build._data = {"actions": []}
    assert build.get_params() == {}


def test_build_get_changeset_empty(build) -> None:
    build._data = {"changeSet": {}}
    assert build.get_changeset_items() == []


def test_build_get_changesets_vcs(build) -> None:
    # This test shall test lines 162-164 in build.py
    build._data = {"changeSets": {"kind": "git"}}
    assert build._get_vcs() == "git"


def test_build_get_number(build) -> None:
    assert build.get_number() == 1


def test_build_get_artifacts(build) -> None:
    afs: List[Artifact] = list(build.get_artifacts())
    assert len(afs) == 1
    assert isinstance(afs[0], Artifact)
    assert list(afs)[0].filename == "foo.txt"


def test_build_get_upstream_job_name(build) -> None:
    assert build.get_upstream_job_name() == "parentBuild"


def test_build_get_ustream_job_name_none(build) -> None:
    build._data = {"actions": []}
    assert build.get_upstream_job_name() is None


def test_build_get_master_job_name(build) -> None:
    assert build.get_master_job_name() == "masterBuild"


def test_build_get_master_build_number(build) -> None:
    assert build.get_master_build_number() == 1



# flake8: noqa
from jenkinsapi import config

JOB_DATA = {
    "actions": [
        {
            "parameterDefinitions": [
                {
                    "defaultParameterValue": {
                        "name": "param1",
                        "value": "test1",
                    },
                    "description": "",
                    "name": "param1",
                    "type": "StringParameterDefinition",
                },
                {
                    "defaultParameterValue": {"name": "param2", "value": ""},
                    "description": "",
                    "name": "param2",
                    "type": "StringParameterDefinition",
                },
            ],
        }
    ],
    "description": "test job",
    "displayName": "foo",
    "displayNameOrNull": None,
    "name": "foo",
    "url": "http://halob:8080/job/foo/",
    "buildable": True,
    "builds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    ],
    # allBuilds is not present in job dict returned by Jenkins
    # it is inserted here to test _add_missing_builds()
    "allBuilds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    ],
    "color": "blue",
    "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    "healthReport": [
        {
            "description": "Build stability: No recent builds failed.",
            "iconUrl": "health-80plus.png",
            "score": 100,
        }
    ],
    "inQueue": False,
    "keepDependencies": False,
    # build running
    "lastBuild": {"number": 4, "url": "http://halob:8080/job/foo/4/"},
    "lastCompletedBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastFailedBuild": None,
    "lastStableBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastSuccessfulBuild": {
        "number": 3,
        "url": "http://halob:8080/job/foo/3/",
    },
    "lastUnstableBuild": None,
    "lastUnsuccessfulBuild": None,
    "nextBuildNumber": 4,
    "property": [],
    "queueItem": None,
    "concurrentBuild": False,
    # test1 job exists, test2 does not
    "downstreamProjects": [{"name": "test1"}, {"name": "test2"}],
    "scm": {},
    "upstreamProjects": [],
}

URL_DATA = {"http://halob:8080/job/foo/%s" % config.JENKINS_API: JOB_DATA}

BUILD_DATA = {
    "actions": [
        {
            "causes": [
                {
                    "shortDescription": "Started by user anonymous",
                    "userId": None,
                    "userName": "anonymous",
                    "upstreamProject": "parentBuild",
                    "upstreamBuild": 1,
                }
            ],
            "parameters": [
                {"name": "masterBuild", "value": "masterBuild"},
                {"name": "lastBuild", "value": 1},
            ],
        }
    ],
    "artifacts": [{"fileName": "foo.txt", "relativePath": "foo.txt"}],
    "building": False,
    "builtOn": "localhost",
    "changeSet": {
        "items": [
            {
                "affectedPaths": ["content/rcm/v00-rcm-xccdf.xml"],
                "author": {
                    "absoluteUrl": "http://jenkins_url/user/username79",
                    "fullName": "username",
                },
                "commitId": "3097",
                "timestamp": 1414398423091,
                "date": "2014-10-27T08:27:03.091288Z",
                "msg": "commit message",
                "paths": [
                    {"editType": "edit", "file": "/some/path/of/changed_file"}
                ],
                "revision": 3097,
                "user": "username",
            }
        ],
        "kind": None,
    },
    "culprits": [],
    "description": "Best build ever!",
    "duration": 5782,
    "estimatedDuration": 106,
    "executor": None,
    "fingerprint": [
        {
            "fileName": "BuildId.json",
            "hash": "e3850a45ab64aa34c1aa66e30c1a8977",
            "original": {"name": "ArtifactGenerateJob", "number": 469},
            "timestamp": 1380270162488,
            "usage": [
                {
                    "name": "test1",
                    "ranges": {"ranges": [{"end": 567, "start": 566}]},
                },
                {
                    "name": "test2",
                    "ranges": {"ranges": [{"end": 150, "start": 139}]},
                },
            ],
        }
    ],
    "fullDisplayName": "foo #1",
    "id": "2013-05-31_23-15-40",
    "keepLog": False,
    "number": 1,
    "result": "SUCCESS",
    "timestamp": 1370042140000,
    "url": "http://localhost:8080/job/foo/1/",
    "runs": [
        {"number": 1, "url": "http//localhost:8080/job/foo/SHARD_NUM=1/1/"},
        {"number": 2, "url": "http//localhost:8080/job/foo/SHARD_NUM=1/2/"},
    ],
}

BUILD_DATA_PIPELINE = {
    "actions": [
        {
            "causes": [
                {
                    "shortDescription": "Started by user anonymous",
                    "userId": None,
                    "userName": "anonymous",
                }
            ]
        }
    ],
    "artifacts": [],
    "building": False,
    "builtOn": "localhost",
    "changeSets": {
        "items": [
            {
                "affectedPaths": ["content/rcm/v00-rcm-xccdf.xml"],
                "author": {
                    "absoluteUrl": "http://jenkins_url/user/username79",
                    "fullName": "username",
                },
                "commitId": "3097",
                "timestamp": 1414398423091,
                "date": "2014-10-27T08:27:03.091288Z",
                "msg": "commit message",
                "paths": [
                    {"editType": "edit", "file": "/some/path/of/changed_file"}
                ],
                "revision": 3097,
                "user": "username",
            }
        ],
        "kind": None,
    },
    "culprits": [],
    "description": "Best build ever!",
    "duration": 5782,
    "estimatedDuration": 106,
    "executor": None,
    "fingerprint": [
        {
            "fileName": "BuildId.json",
            "hash": "e3850a45ab64aa34c1aa66e30c1a8977",
            "original": {"name": "ArtifactGenerateJob", "number": 469},
            "timestamp": 1380270162488,
            "usage": [
                {
                    "name": "test1",
                    "ranges": {"ranges": [{"end": 567, "start": 566}]},
                },
                {
                    "name": "test2",
                    "ranges": {"ranges": [{"end": 150, "start": 139}]},
                },
            ],
        }
    ],
    "fullDisplayName": "foo #1",
    "id": "2013-05-31_23-15-40",
    "keepLog": False,
    "number": 1,
    "result": "SUCCESS",
    "timestamp": 1370042140000,
    "url": "http://localhost:8080/job/foo/1/",
    "runs": [
        {"number": 1, "url": "http//localhost:8080/job/foo/SHARD_NUM=1/1/"},
        {"number": 2, "url": "http//localhost:8080/job/foo/SHARD_NUM=1/2/"},
    ],
}


BUILD_SCM_DATA = {
    "actions": [
        {"causes": [{"shortDescription": "Started by an SCM change"}]},
        {},
        {
            "buildsByBranchName": {
                "origin/HEAD": {
                    "buildNumber": 2,
                    "buildResult": None,
                    "revision": {
                        "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                        "branch": [
                            {
                                "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                                "name": "origin/HEAD",
                            },
                            {
                                "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                                "name": "origin/master",
                            },
                        ],
                    },
                },
                "origin/master": {
                    "buildNumber": 2,
                    "buildResult": None,
                    "revision": {
                        "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                        "branch": [
                            {
                                "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                                "name": "origin/HEAD",
                            },
                            {
                                "SHA1": "d2a5d435fa2df3bff572bd06e43c86544749c5d2",
                                "name": "origin/master",
                            },
                        ],
                    },
                },
                "origin/python_3_compatibility": {
                    "buildNumber": 1,
                    "buildResult": None,
                    "revision": {
                        "SHA1": "c9d1c96bc926ff63a5209c51b3ed537e62ea50e6",
                        "branch": [
                            {
                                "SHA1": "c9d1c96bc926ff63a5209c51b3ed537e62ea50e6",
                                "name": "origin/python_3_compatibility",
                            }
                        ],
                    },
                },
                "origin/unstable": {
                    "buildNumber": 3,
                    "buildResult": None,
                    "revision": {
                        "SHA1": "7def9ed6e92580f37d00e4980c36c4d36e68f702",
                        "branch": [
                            {
                                "SHA1": "7def9ed6e92580f37d00e4980c36c4d36e68f702",
                                "name": "origin/unstable",
                            }
                        ],
                    },
                },
            },
            "lastBuiltRevision": {
                "SHA1": "7def9ed6e92580f37d00e4980c36c4d36e68f702",
                "branch": [
                    {
                        "SHA1": "7def9ed6e92580f37d00e4980c36c4d36e68f702",
                        "name": "origin/unstable",
                    }
                ],
            },
            "remoteUrls": ["https://github.com/salimfadhley/jenkinsapi.git"],
            "scmName": "",
        },
        {},
        {},
    ],
    "artifacts": [],
    "building": False,
    "builtOn": "",
    "changeSet": {"items": [], "kind": "git"},
    "culprits": [],
    "description": None,
    "duration": 1051,
    "estimatedDuration": 2260,
    "executor": None,
    "fullDisplayName": "git_yssrtigfds #3",
    "id": "2013-06-30_01-54-35",
    "keepLog": False,
    "number": 3,
    "result": "SUCCESS",
    "timestamp": 1372553675652,
    "url": "http://localhost:8080/job/git_yssrtigfds/3/",
}

BUILD_ENV_VARS = {
    "_class": "org.jenkinsci.plugins.envinject.EnvInjectVarList",
    "envMap": {"KEY": "VALUE"},
}



import json
import requests


def foo():
    """
    A low level example of how JenkinsAPI runs a parameterized build
    """
    toJson = {"parameter": [{"name": "B", "value": "xyz"}]}
    url = "http://localhost:8080/job/ddd/build"
    # url = 'http://localhost:8000'
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    form = {"json": json.dumps(toJson)}
    response = requests.post(url, data=form, headers=headers)
    print(response.text.encode("UTF-8"))


if __name__ == "__main__":
    foo()



"""
jenkinsapi_tests.test_plugins
"""

import mock

# To run unittests on python 2.6 please use unittest2 library
try:
    import unittest2 as unittest
except ImportError:
    import unittest
try:
    from StringIO import StringIO  # python2
except ImportError:
    from io import BytesIO as StringIO  # python3
import zipfile

from jenkinsapi.jenkins import Requester
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.plugins import Plugins
from jenkinsapi.plugin import Plugin


class TestPlugins(unittest.TestCase):
    DATA = {
        "plugins": [
            {
                "deleted": False,
                "hasUpdate": True,
                "downgradable": False,
                "dependencies": [{}, {}, {}, {}],
                "longName": "Jenkins Subversion Plug-in",
                "active": True,
                "shortName": "subversion",
                "backupVersion": None,
                "url": "http://wiki.jenkins-ci.org/display/"
                "JENKINS/Subversion+Plugin",
                "enabled": True,
                "pinned": False,
                "version": "1.45",
                "supportsDynamicLoad": "MAYBE",
                "bundled": True,
            },
            {
                "deleted": False,
                "hasUpdate": True,
                "downgradable": False,
                "dependencies": [{}, {}],
                "longName": "Maven Integration plugin",
                "active": True,
                "shortName": "maven-plugin",
                "backupVersion": None,
                "url": "http://wiki.jenkins-ci.org/display/JENKINS/"
                "Maven+Project+Plugin",
                "enabled": True,
                "pinned": False,
                "version": "1.521",
                "supportsDynamicLoad": "MAYBE",
                "bundled": True,
            },
        ]
    }

    @mock.patch.object(Jenkins, "_poll")
    def setUp(self, _poll_jenkins):
        _poll_jenkins.return_value = {}

        self.J = Jenkins("http://localhost:8080")

    @mock.patch.object(Plugins, "_poll")
    def test_get_plugins(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        # Can we produce a repr string for this object
        self.assertIsInstance(self.J.get_plugins(), Plugins)

    @mock.patch.object(Plugins, "_poll")
    def test_no_plugins_str(self, _poll_plugins):
        _poll_plugins.return_value = {}

        plugins = self.J.get_plugins()
        self.assertEqual(str(plugins), "[]")

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_str(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertEqual(str(plugins), "['maven-plugin', 'subversion']")

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_len(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertEqual(len(plugins), 2)

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_contains(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertIn("subversion", plugins)
        self.assertIn("maven-plugin", plugins)

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_values(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        p = Plugin(
            {
                "deleted": False,
                "hasUpdate": True,
                "downgradable": False,
                "dependencies": [{}, {}, {}, {}],
                "longName": "Jenkins Subversion Plug-in",
                "active": True,
                "shortName": "subversion",
                "backupVersion": None,
                "url": "http://wiki.jenkins-ci.org/display/JENKINS/"
                "Subversion+Plugin",
                "enabled": True,
                "pinned": False,
                "version": "1.45",
                "supportsDynamicLoad": "MAYBE",
                "bundled": True,
            }
        )

        plugins = self.J.get_plugins().values()
        self.assertIn(p, plugins)

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_keys(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins().keys()
        self.assertIn("subversion", plugins)
        self.assertIn("maven-plugin", plugins)

    @mock.patch.object(Plugins, "_poll")
    def test_plugins_empty(self, _poll_plugins):
        _poll_plugins.return_value = {}

        # list() is required here for python 3.x compatibility
        plugins = list(self.J.get_plugins().keys())
        self.assertEqual([], plugins)

    @mock.patch.object(Plugins, "_poll")
    def test_plugin_get_by_name(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        p = Plugin(
            {
                "deleted": False,
                "hasUpdate": True,
                "downgradable": False,
                "dependencies": [{}, {}, {}, {}],
                "longName": "Jenkins Subversion Plug-in",
                "active": True,
                "shortName": "subversion",
                "backupVersion": None,
                "url": "http://wiki.jenkins-ci.org/display/JENKINS/"
                "Subversion+Plugin",
                "enabled": True,
                "pinned": False,
                "version": "1.45",
                "supportsDynamicLoad": "MAYBE",
                "bundled": True,
            }
        )

        plugin = self.J.get_plugins()["subversion"]
        self.assertEqual(p, plugin)

    @mock.patch.object(Plugins, "_poll")
    def test_get_plugin_details(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA
        plugin = self.J.get_plugins()["subversion"]
        self.assertEqual("1.45", plugin.version)
        self.assertEqual("subversion", plugin.shortName)
        self.assertEqual("Jenkins Subversion Plug-in", plugin.longName)
        self.assertEqual(
            "http://wiki.jenkins-ci.org/display/JENKINS/Subversion+Plugin",
            plugin.url,
        )

    @mock.patch.object(Requester, "post_xml_and_confirm_status")
    def test_install_plugin_bad_input(self, _post):
        with self.assertRaises(ValueError):
            self.J.install_plugin("test")

    @mock.patch.object(Requester, "post_xml_and_confirm_status")
    def test_delete_plugin_bad_input(self, _post):
        with self.assertRaises(ValueError):
            self.J.delete_plugin("test@latest")

    @mock.patch.object(Plugins, "update_center_dict")
    @mock.patch.object(Plugins, "_poll")
    @mock.patch.object(Plugins, "plugin_version_already_installed")
    @mock.patch.object(Plugins, "restart_required")
    @mock.patch.object(Plugins, "_wait_until_plugin_installed")
    @mock.patch.object(Requester, "post_xml_and_confirm_status")
    @mock.patch.object(Jenkins, "safe_restart")
    def test_install_plugin_good_input(
        self,
        _reboot,
        _post,
        _wait,
        _restart_required,
        already_installed,
        _poll_plugins,
        _center_dict,
    ):
        _poll_plugins.return_value = self.DATA
        already_installed.return_value = False
        self.J.install_plugin("test@latest")
        expected_data = '<jenkins> <install plugin="test@latest" /> </jenkins>'
        _post.assert_called_with(
            "/".join(
                [self.J.baseurl, "pluginManager", "installNecessaryPlugins"]
            ),
            data=expected_data,
        )

    @mock.patch.object(Plugins, "update_center_dict")
    @mock.patch.object(Plugins, "_poll")
    @mock.patch.object(Plugins, "plugin_version_already_installed")
    @mock.patch.object(
        Plugins, "restart_required", new_callable=mock.mock.PropertyMock
    )
    @mock.patch.object(Plugins, "_wait_until_plugin_installed")
    @mock.patch.object(Requester, "post_xml_and_confirm_status")
    @mock.patch.object(Jenkins, "safe_restart")
    def test_install_plugins_good_input_no_restart_required(
        self,
        _restart,
        _post,
        _wait,
        restart_required,
        already_installed,
        _poll_plugins,
        _center_dict,
    ):
        _poll_plugins.return_value = self.DATA
        restart_required.return_value = False
        already_installed.return_value = False
        self.J.install_plugins(["test@latest", "test@latest"])
        self.assertEqual(_post.call_count, 2)
        self.assertEqual(_restart.call_count, 0)

    @mock.patch.object(Plugins, "update_center_dict")
    @mock.patch.object(Plugins, "_poll")
    @mock.patch.object(Plugins, "plugin_version_already_installed")
    @mock.patch.object(
        Plugins, "restart_required", new_callable=mock.mock.PropertyMock
    )
    @mock.patch.object(Plugins, "_wait_until_plugin_installed")
    @mock.patch.object(Requester, "post_xml_and_confirm_status")
    @mock.patch.object(Jenkins, "safe_restart")
    def test_install_plugins_good_input_with_restart_required(
        self,
        _restart,
        _post,
        _wait,
        restart_required,
        already_installed,
        _poll_plugins,
        _center_dict,
    ):
        _poll_plugins.return_value = self.DATA
        restart_required.return_value = True
        already_installed.return_value = False
        self.J.install_plugins(["test@latest", "test@latest"])
        self.assertEqual(_post.call_count, 2)
        self.assertEqual(_restart.call_count, 1)

    @mock.patch.object(Plugins, "_poll")
    def test_get_plugin_dependencies(self, _poll_plugins):
        manifest = (
            "Manifest-Version: 1.0\n"
            "bla: somestuff\n"
            "Plugin-Dependencies: aws-java-sdk:1.10.45,aws-credentials:1.15"
        )
        downloaded_plugin = StringIO()
        zipfile.ZipFile(downloaded_plugin, mode="w").writestr(
            "META-INF/MANIFEST.MF", manifest
        )
        _poll_plugins.return_value = self.DATA
        dependencies = self.J.plugins._get_plugin_dependencies(
            downloaded_plugin
        )
        self.assertEqual(len(dependencies), 2)
        for dep in dependencies:
            self.assertIsInstance(dep, Plugin)

    @mock.patch.object(Plugins, "update_center_dict")
    @mock.patch.object(Plugins, "_poll")
    def test_plugin_version_already_installed(self, _poll_plugins, _update):
        _poll_plugins.return_value = self.DATA
        already_installed = Plugin(
            {"shortName": "subversion", "version": "1.45"}
        )
        self.assertTrue(
            self.J.plugins.plugin_version_already_installed(already_installed)
        )
        not_installed = Plugin({"shortName": "subversion", "version": "1.46"})
        self.assertFalse(
            self.J.plugins.plugin_version_already_installed(not_installed)
        )
        latest = Plugin({"shortName": "subversion", "version": "latest"})
        self.assertFalse(
            self.J.plugins.plugin_version_already_installed(latest)
        )

    @mock.patch.object(Plugins, "_poll")
    @mock.patch.object(
        Plugins,
        "update_center_install_status",
        new_callable=mock.mock.PropertyMock,
    )
    def test_restart_required_after_plugin_installation(
        self, status, _poll_plugins
    ):
        _poll_plugins.return_value = self.DATA
        status.return_value = {
            "data": {
                "jobs": [
                    {
                        "installStatus": "SuccessButRequiresRestart",
                        "name": "credentials",
                        "requiresRestart": "true",
                        "title": None,
                        "version": "0",
                    }
                ],
                "state": "RUNNING",
            },
            "status": "ok",
        }
        self.assertTrue(self.J.plugins.restart_required)

    @mock.patch.object(Plugins, "_poll")
    @mock.patch.object(
        Plugins,
        "update_center_install_status",
        new_callable=mock.mock.PropertyMock,
    )
    def test_restart_not_required_after_plugin_installation(
        self, status, _poll_plugins
    ):
        _poll_plugins.return_value = self.DATA
        status.return_value = {
            "data": {"jobs": [], "state": "RUNNING"},
            "status": "ok",
        }
        self.assertFalse(self.J.plugins.restart_required)

    def test_plugin_repr(self):
        p = Plugin(
            {
                "shortName": "subversion",
            }
        )
        self.assertEqual(repr(p), "<jenkinsapi.plugin.Plugin subversion>")


if __name__ == "__main__":
    unittest.main()



"""
Start a Parameterized Build
"""

from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins("http://localhost:8080")

params = {"VERSION": "1.2.3", "PYTHON_VER": "2.7"}

# This will start the job in non-blocking manner
jenkins.build_job("foo", params)


# This will start the job and will return a QueueItem object which
# can be used to get build results
job = jenkins["foo"]
qi = job.invoke(build_params=params)

# Block this script until build is finished
if qi.is_queued() or qi.is_running():
    qi.block_until_complete()

build = qi.get_build()
print(build)



"""
System tests for `jenkinsapi.jenkins` module.
"""

import io
import time

from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import JOB_WITH_FILE
from jenkinsapi_tests.systests.job_configs import JOB_WITH_FILE_AND_PARAMS
from jenkinsapi_tests.systests.job_configs import JOB_WITH_PARAMETERS


def test_invoke_job_with_file(jenkins):
    file_data = random_string()
    param_file = io.BytesIO(file_data.encode("utf-8"))

    job_name = "create1_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_FILE)

    assert job.has_params() is True
    assert len(job.get_params_list()) != 0

    job.invoke(block=True, files={"file.txt": param_file})

    build = job.get_last_build()
    while build.is_running():
        time.sleep(0.25)

    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True
    art_file = artifacts["file.txt"]
    assert art_file.get_data().decode("utf-8").strip() == file_data


def test_invoke_job_parameterized(jenkins):
    param_B = random_string()

    job_name = "create2_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_PARAMETERS)
    job.invoke(block=True, build_params={"B": param_B})
    build = job.get_last_build()

    artifacts = build.get_artifact_dict()
    artB = artifacts["b.txt"]
    assert artB.get_data().decode("UTF-8", "replace").strip() == param_B

    assert param_B in build.get_console()


def test_parameterized_job_build_queuing(jenkins):
    """
    Accept multiple builds of parameterized jobs with unique parameters.
    """
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_PARAMETERS)

    # Latest Jenkins schedules builds to run right away, so remove all
    # executors from master node to investigate queue
    master = jenkins.nodes["Built-In Node"]
    num_executors = master.get_num_executors()
    master.set_num_executors(0)

    for i in range(3):
        param_B = random_string()
        params = {"B": param_B}
        job.invoke(build_params=params)

    assert job.has_queued_build(params) is True

    master.set_num_executors(num_executors)

    while job.has_queued_build(params):
        time.sleep(0.25)

    build = job.get_last_build()
    while build.is_running():
        time.sleep(0.25)

    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True
    artB = artifacts["b.txt"]
    assert artB.get_data().decode("utf-8").strip() == param_B

    assert param_B in build.get_console()


def test_parameterized_multiple_builds_get_the_same_queue_item(jenkins):
    """
    Multiple attempts to run the same parameterized
    build will get the same queue item.
    """
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_PARAMETERS)

    # Latest Jenkins schedules builds to run right away, so remove all
    # executors from master node to investigate queue
    master = jenkins.nodes["Built-In Node"]
    num_executors = master.get_num_executors()
    master.set_num_executors(0)

    for i in range(3):
        params = {"B": random_string()}
        qq0 = job.invoke(build_params=params)

    qq1 = job.invoke(build_params=params)
    assert qq0 == qq1

    master.set_num_executors(num_executors)


def test_invoke_job_with_file_and_params(jenkins):
    file_data = random_string()
    param_data = random_string()
    param_file = io.BytesIO(file_data.encode("utf-8"))

    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_FILE_AND_PARAMS)

    assert job.has_params() is True
    assert len(job.get_params_list()) != 0

    qi = job.invoke(
        block=True,
        files={"file.txt": param_file},
        build_params={"B": param_data},
    )

    build = qi.get_build()
    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True
    art_file = artifacts["file.txt"]
    assert art_file.get_data().decode("utf-8").strip() == file_data
    art_param = artifacts["file1.txt"]
    assert art_param.get_data().decode("utf-8").strip() == param_data



"""
A lower-level implementation of copying a job in Jenkins
"""

import requests
from pkg_resources import resource_string
from jenkinsapi.jenkins import Jenkins
from jenkinsapi_tests.test_utils.random_strings import random_string

J = Jenkins("http://localhost:8080")
jobName = random_string()
jobName2 = "%s_2" % jobName

url = "http://localhost:8080/createItem?from=%s&name=%s&mode=copy" % (
    jobName,
    jobName2,
)

xml = resource_string("examples", "addjob.xml")
j = J.create_job(jobname=jobName, xml=xml)


h = {"Content-Type": "application/x-www-form-urlencoded"}
response = requests.post(url, data="dysjsjsjs", headers=h)
print(response.text.encode("UTF-8"))



"""
Module for jenkinsapi Executer class
"""

from __future__ import annotations

from jenkinsapi.jenkinsbase import JenkinsBase
import logging

log = logging.getLogger(__name__)


class Executor(JenkinsBase):
    """
    Class to hold information on nodes that are attached as slaves to the
    master jenkins instance
    """

    def __init__(
        self, baseurl: str, nodename: str, jenkins_obj: "Jenkins", number: int
    ) -> None:
        """
        Init a node object by providing all relevant pointers to it
        :param baseurl: basic url for querying information on a node
        :param nodename: hostname of the node
        :param jenkins_obj: ref to the jenkins obj
        :return: Node obj
        """
        self.nodename: str = nodename
        self.number: int = number
        self.jenkins: "Jenkins" = jenkins_obj
        self.baseurl: str = baseurl
        JenkinsBase.__init__(self, baseurl)

    def __str__(self) -> str:
        return f"{self.nodename} {self.number}"

    def get_jenkins_obj(self) -> "Jenkins":
        return self.jenkins

    def get_progress(self) -> str:
        """Returns percentage"""
        return self.poll(tree="progress")["progress"]

    def get_number(self) -> int:
        """
        Get Executor number.
        """
        return self.poll(tree="number")["number"]

    def is_idle(self) -> bool:
        """
        Returns Boolean: whether Executor is idle or not.
        """
        return self.poll(tree="idle")["idle"]

    def likely_stuck(self) -> bool:
        """
        Returns Boolean: whether Executor is likely stuck or not.
        """
        return self.poll(tree="likelyStuck")["likelyStuck"]

    def get_current_executable(self) -> str:
        """
        Returns the current Queue.Task this executor is running.
        """
        return self.poll(tree="currentExecutable")["currentExecutable"]



"""
System tests for `jenkinsapi.jenkins` module.
"""

import time
import logging
from jenkinsapi_tests.systests.job_configs import LONG_RUNNING_JOB
from jenkinsapi_tests.test_utils.random_strings import random_string

log = logging.getLogger(__name__)


def test_get_executors(jenkins):
    node_name = random_string()
    node_dict = {
        "num_executors": 2,
        "node_description": "Test JNLP Node",
        "remote_fs": "/tmp",
        "labels": "systest_jnlp",
        "exclusive": True,
    }
    jenkins.nodes.create_node(node_name, node_dict)
    executors = jenkins.get_executors(node_name)
    assert executors.count == 2

    for count, execs in enumerate(executors):
        assert count == execs.get_number()
        assert execs.is_idle() is True


def test_running_executor(jenkins):
    node_name = random_string()
    node_dict = {
        "num_executors": 1,
        "node_description": "Test JNLP Node",
        "remote_fs": "/tmp",
        "labels": "systest_jnlp",
        "exclusive": True,
    }
    jenkins.nodes.create_node(node_name, node_dict)
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, LONG_RUNNING_JOB)
    qq = job.invoke()
    qq.block_until_building()

    if job.is_running() is False:
        time.sleep(1)
    executors = jenkins.get_executors(node_name)
    all_idle = True
    for execs in executors:
        if execs.is_idle() is False:
            all_idle = False
            assert execs.get_progress() != -1
            assert execs.get_current_executable() == qq.get_build_number()
            assert execs.likely_stuck() is False
    assert all_idle is True, "Executor should have been triggered."


def test_idle_executors(jenkins):
    node_name = random_string()
    node_dict = {
        "num_executors": 1,
        "node_description": "Test JNLP Node",
        "remote_fs": "/tmp",
        "labels": "systest_jnlp",
        "exclusive": True,
    }
    jenkins.nodes.create_node(node_name, node_dict)
    executors = jenkins.get_executors(node_name)

    for execs in executors:
        assert execs.get_progress() == -1
        assert execs.get_current_executable() is None
        assert execs.likely_stuck() is False
        assert execs.is_idle() is True



"""
System tests for `jenkinsapi.plugins` module.
"""

import logging
import pytest
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi.plugin import Plugin

log = logging.getLogger(__name__)


def test_plugin_data(jenkins):
    # It takes time to get plugins json from remote
    timeout = jenkins.requester.timeout
    jenkins.requester.timeout = 60
    jenkins.plugins.check_updates_server()
    jenkins.requester.timeout = timeout

    assert "workflow-api" in jenkins.plugins


def test_get_missing_plugin(jenkins):
    plugins = jenkins.get_plugins()
    with pytest.raises(KeyError):
        plugins["lsdajdaslkjdlkasj"]  # this plugin surely does not exist!


def test_get_single_plugin(jenkins):
    plugins = jenkins.get_plugins()
    plugin_name, plugin = next(plugins.iteritems())

    assert isinstance(plugin_name, str)
    assert isinstance(plugin, Plugin)


def test_get_single_plugin_depth_2(jenkins):
    plugins = jenkins.get_plugins(depth=2)
    _, plugin = next(plugins.iteritems())

    assert isinstance(plugin, Plugin)


def test_delete_inexistant_plugin(jenkins):
    with pytest.raises(KeyError):
        del jenkins.plugins[random_string()]


#    def test_install_uninstall_plugin(jenkins):
#        plugin_name = "suppress-stack-trace"
#
#        plugin_dict = {
#            "shortName": plugin_name,
#            "version": "latest",
#        }
#        jenkins.plugins[plugin_name] = Plugin(plugin_dict)
#
#        assert plugin_name in jenkins.plugins
#
#        plugin = jenkins.get_plugins()[plugin_name]
#        assert isinstance(plugin, Plugin)
#        assert plugin.shortName == plugin_name
#
#        del jenkins.plugins[plugin_name]
#        assert jenkins.plugins[plugin_name].deleted
#
#
#    def test_install_multiple_plugins(jenkins):
#        plugin_one_name = "keyboard-shortcuts-plugin"
#        plugin_one_version = "latest"
#        plugin_one = "@".join((plugin_one_name, plugin_one_version))
#        plugin_two = Plugin(
#            {"shortName": "emotional-jenkins-plugin", "version": "latest"}
#        )
#
#        assert isinstance(plugin_two, Plugin)
#
#        plugin_list = [plugin_one, plugin_two]
#
#        jenkins.install_plugins(plugin_list)
#
#        assert plugin_one_name in jenkins.plugins
#        assert plugin_two.shortName in jenkins.plugins
#
#        del jenkins.plugins["keyboard-shortcuts-plugin"]
#        del jenkins.plugins["emotional-jenkins-plugin"]
#
#
#    def test_downgrade_plugin(jenkins):
#        plugin_name = "console-badge"
#        plugin_version = "latest"
#        plugin = Plugin({"shortName": plugin_name, "version": plugin_version})
#
#        assert isinstance(plugin, Plugin)
#
#        # Need to restart when not installing the latest version
#        jenkins.install_plugins([plugin])
#
#        installed_plugin = jenkins.plugins[plugin_name]
#
#        assert installed_plugin.version == "1.1"
#
#        older_plugin = Plugin({"shortName": plugin_name, "version": "1.0"})
#        jenkins.install_plugins(
#            [older_plugin], restart=True, wait_for_reboot=True)
#        installed_older_plugin = jenkins.plugins[plugin_name]
#
#        assert installed_older_plugin.version == "1.0"
#
#        del jenkins.plugins[plugin_name]



"""
System tests for `jenkinsapi.jenkins` module.
"""

import logging
import pytest
from jenkinsapi.node import Node
from jenkinsapi.credential import SSHKeyCredential
from jenkinsapi_tests.test_utils.random_strings import random_string

log = logging.getLogger(__name__)

TOOL_KEY = "hudson.tasks.Maven$MavenInstallation$DescriptorImpl@Maven 3.0.5"


def test_online_offline(jenkins):
    """
    Can we flip the online / offline state of the master node.
    """
    # Master node name should be case insensitive
    # mn0 = jenkins.get_node('MaStEr')
    mn = jenkins.get_node("Built-In Node")
    # self.assertEqual(mn, mn0)

    mn.set_online()  # It should already be online, hence no-op
    assert mn.is_online() is True

    mn.set_offline()  # We switch that suckah off
    mn.set_offline()  # This should be a no-op
    assert mn.is_online() is False

    mn.set_online()  # Switch it back on
    assert mn.is_online() is True


def test_create_jnlp_node(jenkins):
    node_name = random_string()
    node_dict = {
        "num_executors": 1,
        "node_description": "Test JNLP Node",
        "remote_fs": "/tmp",
        "labels": "systest_jnlp",
        "exclusive": True,
        "tool_location": [
            {
                "key": TOOL_KEY,
                "home": "/home/apache-maven-3.0.5/",
            },
        ],
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert isinstance(node, Node) is True

    del jenkins.nodes[node_name]


def test_create_ssh_node(jenkins):
    node_name = random_string()
    creds = jenkins.get_credentials()

    cred_descr = random_string()
    cred_dict = {
        "description": cred_descr,
        "userName": "username",
        "passphrase": "",
        "private_key": "-----BEGIN RSA PRIVATE KEY-----",
    }
    creds[cred_descr] = SSHKeyCredential(cred_dict)
    node_dict = {
        "num_executors": 1,
        "node_description": "Description %s" % node_name,
        "remote_fs": "/tmp",
        "labels": node_name,
        "exclusive": False,
        "host": "127.0.0.1",
        "port": 22,
        "credential_description": cred_descr,
        "jvm_options": "",
        "java_path": "",
        "prefix_start_slave_cmd": "",
        "suffix_start_slave_cmd": "",
        "retention": "ondemand",
        "ondemand_delay": 0,
        "ondemand_idle_delay": 5,
        "tool_location": [
            {
                "key": TOOL_KEY,
                "home": "/home/apache-maven-3.0.5/",
            },
        ],
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert isinstance(node, Node) is True
    del jenkins.nodes[node_name]

    jenkins.nodes[node_name] = node_dict
    assert isinstance(jenkins.nodes[node_name], Node) is True
    del jenkins.nodes[node_name]


def test_delete_node(jenkins):
    node_name = random_string()
    node_dict = {
        "num_executors": 1,
        "node_description": "Test JNLP Node",
        "remote_fs": "/tmp",
        "labels": "systest_jnlp",
        "exclusive": True,
    }
    jenkins.nodes.create_node(node_name, node_dict)
    del jenkins.nodes[node_name]

    with pytest.raises(KeyError):
        jenkins.nodes[node_name]

    with pytest.raises(KeyError):
        del jenkins.nodes["not_exist"]


def test_delete_all_nodes(jenkins):
    nodes = jenkins.nodes

    for name in nodes.keys():
        del nodes[name]

    assert len(jenkins.nodes) == 1


def test_get_node_labels(jenkins):
    node_name = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert node.get_labels() == node_labels

    del jenkins.nodes[node_name]


def test_add_node_labels(jenkins):
    node_name = random_string()
    extra_label = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    node.add_labels(extra_label)
    assert node.get_labels() == " ".join([node_labels, extra_label])

    del jenkins.nodes[node_name]


def test_add_list_node_labels(jenkins):
    node_name = random_string()
    extra_labels = [random_string(), random_string()]
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    node.add_labels(extra_labels)
    labels_on_node = node.get_labels().split(" ")
    assert set(extra_labels + node_labels.split(" ")) == set(labels_on_node)

    del jenkins.nodes[node_name]


def test_modify_node_labels(jenkins):
    node_name = random_string()
    extra_label = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    node.modify_labels(extra_label)
    assert node.get_labels() == extra_label

    del jenkins.nodes[node_name]


def test_delete_list_node_labels(jenkins):
    node_name = random_string()
    extra_labels = [random_string(), random_string()]
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    node.add_labels(extra_labels)
    labels_on_node = node.get_labels().split(" ")

    assert set(extra_labels + node_labels.split(" ")) == set(labels_on_node)
    node.delete_labels(extra_labels)

    labels_on_node = node.get_labels().split(" ")
    assert set(node_labels.split(" ")) == set(labels_on_node)

    del jenkins.nodes[node_name]


def test_dryrun_delete_list_node_labels(jenkins):
    node_name = random_string()
    extra_labels = [random_string(), random_string()]
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    node.add_labels(extra_labels)
    labels_on_node = node.get_labels().split(" ")

    assert set(extra_labels + node_labels.split(" ")) == set(labels_on_node)
    node.delete_labels(extra_labels, dryRun=True)

    labels_on_node = node.get_labels().split(" ")
    assert set(extra_labels + node_labels.split(" ")) == set(labels_on_node)

    del jenkins.nodes[node_name]


def test_get_executors(jenkins):
    node_name = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    with pytest.raises(AttributeError):
        assert node.get_config_element("executors") == "1"

    assert node.get_config_element("numExecutors") == "1"

    del jenkins.nodes[node_name]


def test_set_executors(jenkins):
    node_name = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    assert node.set_config_element("numExecutors", "5") is None

    assert node.get_config_element("numExecutors") == "5"

    del jenkins.nodes[node_name]


def test_set_master_executors(jenkins):
    node = jenkins.nodes["Built-In Node"]

    assert node.get_num_executors() == 2

    node.set_num_executors(5)
    assert node.get_num_executors() == 5

    node.set_num_executors(2)


def test_offline_reason(jenkins):
    node_name = random_string()
    node_labels = "LABEL1 LABEL2"
    node_dict = {
        "num_executors": 1,
        "node_description": "Test Node with Labels",
        "remote_fs": "/tmp",
        "labels": node_labels,
        "exclusive": True,
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    node.toggle_temporarily_offline("test1")
    node.poll()
    assert node.offline_reason() == "test1"

    node.update_offline_reason("test2")
    node.poll()
    assert node.offline_reason() == "test2"

    del jenkins.nodes[node_name]



import unittest.mock as mock

import pytest

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.view import View
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import NotFound

DATA = {
    "description": "Important Shizz",
    "jobs": [
        {"color": "blue", "name": "foo", "url": "http://halob:8080/job/foo/"},
        {
            "color": "red",
            "name": "test_jenkinsapi",
            "url": "http://halob:8080/job/test_jenkinsapi/",
        },
    ],
    "name": "FodFanFo",
    "property": [],
    "url": "http://halob:8080/view/FodFanFo/",
}

JOB_DATA = {
    "actions": [],
    "description": "test job",
    "displayName": "foo",
    "displayNameOrNull": None,
    "name": "foo",
    "url": "http://halob:8080/job/foo/",
    "buildable": True,
    "builds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    ],
    "color": "blue",
    "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    "healthReport": [
        {
            "description": "Build stability: No recent builds failed.",
            "iconUrl": "health-80plus.png",
            "score": 100,
        }
    ],
    "inQueue": False,
    "keepDependencies": False,
    "lastBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastCompletedBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastFailedBuild": None,
    "lastStableBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastSuccessfulBuild": {
        "number": 3,
        "url": "http://halob:8080/job/foo/3/",
    },
    "lastUnstableBuild": None,
    "lastUnsuccessfulBuild": None,
    "nextBuildNumber": 4,
    "property": [],
    "queueItem": None,
    "concurrentBuild": False,
    "downstreamProjects": [],
    "scm": {},
    "upstreamProjects": [],
}


@pytest.fixture
def jenkins():
    jenkins = mock.MagicMock(autospec=True)
    jenkins.has_job.return_value = False
    return jenkins


@pytest.fixture
@mock.patch.object(Job, "_poll", autospec=True)
@mock.patch.object(View, "_poll", autospec=True)
def view(_view_poll, _job_poll, jenkins):
    _view_poll.return_value = DATA
    _job_poll.return_value = JOB_DATA
    return View("http://localhost:800/view/FodFanFo", "FodFanFo", jenkins)


@pytest.fixture
def jenkins_patch():
    class Jenkins:
        def has_job(self, job_name):
            return False

        def get_jenkins_obj_from_url(self, url):
            return self

    return Jenkins


@pytest.fixture
def busy_patch():
    class Jenkins:
        def has_job(self, job_name):
            return True

        def get_jenkins_obj_from_url(self, url):
            return self

    return Jenkins


class TestView:
    def test_returns_name_when_repr_is_called(self, view):
        assert repr(view) == "FodFanFo"

    def test_returns_name_when_str_method_called(self, view):
        assert str(view) == "FodFanFo"

    def test_raises_error_when_is_called(self, view):
        with pytest.raises(AttributeError):
            view.id()

    def test_returns_name_when_name_property_is_called(self, view):
        assert view.name == "FodFanFo"

    @mock.patch.object(JenkinsBase, "_poll")
    def test_iteritems(self, _poll, view):
        _poll.return_value = JOB_DATA
        for job_name, job_obj in view.iteritems():
            assert isinstance(job_obj, Job)
            assert job_name in ["foo", "test_jenkinsapi"]

    def test_returns_dict_of_job_info_when_job_dict_method_called(self, view):
        jobs = view.get_job_dict()

        assert jobs == {
            "foo": "http://halob:8080/job/foo/",
            "test_jenkinsapi": "http://halob:8080/job/test_jenkinsapi/",
        }

    def test_returns_len_when_len_is_called(self, view):
        assert len(view) == 2

    @mock.patch.object(JenkinsBase, "_poll")
    def test_getitem(self, _poll, view):
        _poll.return_value = JOB_DATA

        assert isinstance(view["foo"], Job)

    def test_sets_delete_to_true_when_deleted(self, view):
        view.delete()

        assert view.deleted

    def test_returns_url_when_get_job_url_is_called(self, view):
        url = view.get_job_url("foo")

        assert url == "http://halob:8080/job/foo/"

    def test_raises_not_found_when_get_job_url_is_invalid(self, view):
        with pytest.raises(NotFound):
            view.get_job_url("bar")

    @mock.patch.object(View, "get_jenkins_obj")
    def test_returns_false_when_adding_wrong_job(
        self, _get_jenkins, view, jenkins_patch
    ):
        _get_jenkins.return_value = jenkins_patch()
        result = view.add_job("bar")

        assert result is False

    def test_returns_false_when_add_existing_job(self, view):
        result = view.add_job("foo")

        assert result is False

    def test_get_nested_view_dict(self, view):
        result = view.get_nested_view_dict()

        assert isinstance(result, dict)

    def test_returns_jenkins_obj_when_get_jenkins_obj_is_called(self, view):
        obj = view.get_jenkins_obj()

        assert obj == view.jenkins_obj


class TestKeys:
    def test_returns_key_when_called(self, view):
        keys = view.keys()

        assert "foo" in list(keys)
        assert "test_jenkinsapi" in list(keys)


class TestAddJob:
    @mock.patch.object(JenkinsBase, "_poll")
    def test_returns_true_when_no_job_provided(self, _poll, view):
        _poll.return_value = DATA

        result = view.add_job("bar")

        assert result is True

    @mock.patch.object(JenkinsBase, "_poll")
    def test_returns_false_when_already_registered(self, _poll, view):
        _poll.return_value = DATA

        result = view.add_job("foo")

        assert result is False

    @mock.patch.object(View, "get_jenkins_obj")
    def test_returns_false_when_jenkins_has_job(
        self, _get_jenkins, view, jenkins_patch
    ):
        _get_jenkins.return_value = jenkins_patch()

        result = view.add_job("Foo")

        _get_jenkins.assert_called()
        assert result is False

    @mock.patch.object(View, "get_jenkins_obj")
    @mock.patch.object(JenkinsBase, "_poll")
    def test_returns_true_when_jenkins_has_job(
        self, _poll, _get_jenkins, view, jenkins
    ):
        _get_jenkins.return_value = jenkins()
        _poll.return_value = DATA
        result = view.add_job("Foo")

        _get_jenkins.assert_called()
        assert result is True


class TestRemove:
    @mock.patch.object(JenkinsBase, "_poll")
    def test_returns_true_when_job_has_been_removed(self, _poll, view):
        _poll.return_value = DATA

        result = view.remove_job("foo")

        assert result is True

    @mock.patch.object(JenkinsBase, "_poll")
    def test_returns_false_when_job_does_not_exist(self, _poll, view):
        _poll.return_value = DATA

        result = view.remove_job("Non-existant Foo")

        assert result is False



"""
Example of using CrumbRequester - when CSRF protection is enabled in Jenkins
"""

from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins(
    "http://localhost:8080",
    username="admin",
    password="password",
    use_crumb=True,
)

for job_name in jenkins.jobs:
    print(job_name)



import pytest
from mock import Mock, patch, call
from requests.exceptions import HTTPError
from jenkinsapi.artifact import Artifact
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.fingerprint import Fingerprint
from jenkinsapi.custom_exceptions import ArtifactBroken

try:
    import unittest2 as unittest
except ImportError:
    import unittest


@pytest.fixture()
def artifact(mocker):
    return Artifact(
        "artifact.zip",
        "http://foo/job/TestJob/1/artifact/artifact.zip",
        mocker.MagicMock(),
    )


def test_verify_download_valid_positive(artifact, monkeypatch):
    def fake_md5(cls, fspath):  # pylint: disable=unused-argument
        return "097c42989a9e5d9dcced7b35ec4b0486"

    monkeypatch.setattr(Artifact, "_md5sum", fake_md5)

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)

    def fake_validate(cls, filename, job, build):  # pylint: disable=unused-argument  # pylint: disable=unused-argument
        return True

    monkeypatch.setattr(Fingerprint, "validate_for_build", fake_validate)

    assert artifact._verify_download(
        "/tmp/artifact.zip", strict_validation=False
    )


def test_verify_download_valid_positive_with_rename(artifact, monkeypatch):
    def fake_md5(cls, fspath):  # pylint: disable=unused-argument
        return "097c42989a9e5d9dcced7b35ec4b0486"

    monkeypatch.setattr(Artifact, "_md5sum", fake_md5)

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)

    def fake_validate(cls, filename, job, build):  # pylint: disable=unused-argument  # pylint: disable=unused-argument
        return filename == "artifact.zip"

    monkeypatch.setattr(Fingerprint, "validate_for_build", fake_validate)

    assert artifact._verify_download(
        "/tmp/temporary_filename", strict_validation=False
    )


def test_verify_download_valid_negative(artifact, monkeypatch):
    def fake_md5(cls, fspath):  # pylint: disable=unused-argument
        return "097c42989a9e5d9dcced7b35ec4b0486"

    monkeypatch.setattr(Artifact, "_md5sum", fake_md5)

    class FakeResponse(object):
        status_code = 404
        text = "{}"

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)

    def fake_validate(cls, filename, job, build):  # pylint: disable=unused-argument  # pylint: disable=unused-argument
        return True

    monkeypatch.setattr(Fingerprint, "validate_for_build", fake_validate)

    assert artifact._verify_download(
        "/tmp/artifact.zip", strict_validation=False
    )


def test_verify_dl_valid_negative_strict(artifact, monkeypatch):
    def fake_md5(cls, fspath):  # pylint: disable=unused-argument
        return "097c42989a9e5d9dcced7b35ec4b0486"

    monkeypatch.setattr(Artifact, "_md5sum", fake_md5)

    class FakeResponse(object):
        status_code = 404
        text = "{}"

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)

    with pytest.raises(ArtifactBroken) as ab:
        artifact._verify_download("/tmp/artifact.zip", strict_validation=True)

    assert (
        "Artifact 097c42989a9e5d9dcced7b35ec4b0486 seems to be broken"
        in str(ab.value)
    )


def test_verify_download_invalid(artifact, monkeypatch):
    def fake_md5(cls, fspath):  # pylint: disable=unused-argument
        return "097c42989a9e5d9dcced7b35ec4b0486"

    monkeypatch.setattr(Artifact, "_md5sum", fake_md5)

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)

    def fake_validate(cls, filename, job, build):  # pylint: disable=unused-argument  # pylint: disable=unused-argument
        return False

    monkeypatch.setattr(Fingerprint, "validate_for_build", fake_validate)

    with pytest.raises(ArtifactBroken) as ab:
        artifact._verify_download("/tmp/artifact.zip", strict_validation=True)

    assert (
        "Artifact 097c42989a9e5d9dcced7b35ec4b0486 seems to be broken"
        in str(ab.value)
    )


class ArtifactTest(unittest.TestCase):
    def setUp(self):
        self._build = build = Mock()
        build.buildno = 9999
        job = self._build.job
        job.jenkins.baseurl = "http://localhost"
        job.name = "TestJob"
        self._artifact = Artifact(
            "artifact.zip",
            "http://localhost/job/TestJob/9999/artifact/artifact.zip",
            build,
        )

    @patch("jenkinsapi.artifact.os.path.exists", spec=True, return_value=True)
    def test_save_has_valid_local_copy(self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(return_value=True)

        assert artifact.save("/tmp/artifact.zip") == "/tmp/artifact.zip"

        mock_exists.assert_called_once_with("/tmp/artifact.zip")
        artifact._verify_download.assert_called_once_with(
            "/tmp/artifact.zip", False
        )

    @patch("jenkinsapi.artifact.os.path.exists", spec=True, return_value=True)
    def test_save_has_invalid_local_copy_dl_again(self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(side_effect=[ArtifactBroken, True])
        artifact._do_download = Mock(return_value="/tmp/artifact.zip")

        assert artifact.save("/tmp/artifact.zip", True) == "/tmp/artifact.zip"

        mock_exists.assert_called_once_with("/tmp/artifact.zip")
        artifact._do_download.assert_called_once_with("/tmp/artifact.zip")
        assert (
            artifact._verify_download.mock_calls
            == [call("/tmp/artifact.zip", True)] * 2
        )

    @patch("jenkinsapi.artifact.os.path.exists", spec=True, return_value=True)
    def test_has_invalid_lcl_copy_dl_but_invalid(self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(
            side_effect=[ArtifactBroken, ArtifactBroken]
        )
        artifact._do_download = Mock(return_value="/tmp/artifact.zip")

        with pytest.raises(ArtifactBroken):
            artifact.save("/tmp/artifact.zip", True)

        mock_exists.assert_called_once_with("/tmp/artifact.zip")
        artifact._do_download.assert_called_once_with("/tmp/artifact.zip")
        assert (
            artifact._verify_download.mock_calls
            == [call("/tmp/artifact.zip", True)] * 2
        )

    @patch("jenkinsapi.artifact.os.path.exists", spec=True, return_value=False)
    def test_save_has_no_local_copy(self, mock_exists):
        artifact = self._artifact
        artifact._do_download = Mock(return_value="/tmp/artifact.zip")
        artifact._verify_download = Mock(return_value=True)

        assert artifact.save("/tmp/artifact.zip") == "/tmp/artifact.zip"

        mock_exists.assert_called_once_with("/tmp/artifact.zip")
        artifact._do_download.assert_called_once_with("/tmp/artifact.zip")
        artifact._verify_download.assert_called_once_with(
            "/tmp/artifact.zip", False
        )



"""
System tests for `jenkinsapi.jenkins` module.
"""

import logging
from jenkinsapi.view import View
from jenkinsapi.views import Views
from jenkinsapi.job import Job
from jenkinsapi.api import get_view_from_url
from jenkinsapi_tests.systests.job_configs import EMPTY_JOB
from jenkinsapi_tests.systests.view_configs import VIEW_WITH_FILTER_AND_REGEX
from jenkinsapi_tests.test_utils.random_strings import random_string

log = logging.getLogger(__name__)


def create_job(jenkins, job_name="whatever"):
    job = jenkins.create_job(job_name, EMPTY_JOB)
    return job


def test_make_views(jenkins):
    view_name = random_string()
    assert view_name not in jenkins.views
    new_view = jenkins.views.create(view_name)
    assert view_name in jenkins.views
    assert isinstance(new_view, View) is True
    assert view_name == str(new_view)

    # Can we create a view that already exists?
    existing = jenkins.views.create(view_name)
    assert existing == new_view

    # Can we use the API convenience methods
    new_view_1 = get_view_from_url(new_view.baseurl)
    assert new_view == new_view_1

    del jenkins.views[view_name]


def test_add_job_to_view(jenkins):
    job_name = random_string()
    create_job(jenkins, job_name)

    view_name = random_string()
    assert view_name not in jenkins.views
    new_view = jenkins.views.create(view_name)
    assert view_name in jenkins.views
    assert isinstance(new_view, View) is True

    assert job_name not in new_view
    assert new_view.add_job(job_name) is True
    assert job_name in new_view
    assert isinstance(new_view[job_name], Job) is True

    assert len(new_view) == 1
    for j_name, j in new_view.iteritems():
        assert j_name == job_name
        assert isinstance(j, Job) is True

    for j in new_view.values():
        assert isinstance(j, Job) is True

    jobs = new_view.items()
    assert isinstance(jobs, list) is True
    assert isinstance(jobs[0], tuple) is True

    assert new_view.add_job(job_name) is False
    assert new_view.add_job("unknown") is False

    del jenkins.views[view_name]


def test_create_and_delete_views(jenkins):
    view1_name = random_string()
    new_view = jenkins.views.create(view1_name)
    assert isinstance(new_view, View) is True
    assert view1_name in jenkins.views
    del jenkins.views[view1_name]
    assert view1_name not in jenkins.views


def test_create_and_delete_views_by_url(jenkins):
    view1_name = random_string()
    new_view = jenkins.views.create(view1_name)
    assert isinstance(new_view, View) is True
    assert view1_name in jenkins.views

    view_url = new_view.baseurl
    view_by_url = jenkins.get_view_by_url(view_url)
    assert isinstance(view_by_url, View) is True
    jenkins.delete_view_by_url(view_url)

    assert view1_name not in jenkins.views


def test_delete_view_which_does_not_exist(jenkins):
    view1_name = random_string()
    assert view1_name not in jenkins.views
    del jenkins.views[view1_name]


def test_update_view_config(jenkins):
    view_name = random_string()
    new_view = jenkins.views.create(view_name)
    assert isinstance(new_view, View) is True
    assert view_name in jenkins.views

    config = jenkins.views[view_name].get_config().strip()
    new_view_config = VIEW_WITH_FILTER_AND_REGEX % view_name
    assert config != new_view_config

    jenkins.views[view_name].update_config(new_view_config)
    config = jenkins.views[view_name].get_config().strip()
    assert config == new_view_config


def test_make_nested_views(jenkins):
    job = create_job(jenkins)
    top_view_name = random_string()
    sub1_view_name = random_string()
    sub2_view_name = random_string()

    assert top_view_name not in jenkins.views
    tv = jenkins.views.create(top_view_name, Views.NESTED_VIEW)
    assert top_view_name in jenkins.views
    assert isinstance(tv, View) is True

    # Empty sub view
    sv1 = tv.views.create(sub1_view_name)
    assert sub1_view_name in tv.views
    assert isinstance(sv1, View) is True

    # Sub view with job in it
    tv.views[sub2_view_name] = job.name
    assert sub2_view_name in tv.views
    sv2 = tv.views[sub2_view_name]
    assert isinstance(sv2, View) is True
    assert job.name in sv2

    # Can we use the API convenience methods
    new_view = get_view_from_url(sv2.baseurl)
    assert new_view == sv2


def test_add_to_view_after_copy(jenkins):
    # This test is for issue #291
    job = create_job(jenkins)
    new_job_name = random_string()
    view_name = random_string()
    new_view = jenkins.views.create(view_name)
    new_view = jenkins.views[view_name]
    new_job = jenkins.copy_job(job.name, new_job_name)
    assert new_view.add_job(new_job.name) is True
    assert new_job.name in new_view


def test_get_job_config(jenkins):
    # This test is for issue #301
    job = create_job(jenkins)
    view_name = random_string()
    new_view = jenkins.views.create(view_name)

    assert new_view.add_job(job.name) is True

    assert "<?xml" in jenkins.get_job(job.name).get_config()
    for _, job in jenkins.views[view_name].items():
        assert "<?xml" in job.get_config()


def test_remove_job_from_view(jenkins):
    job_name = random_string()
    create_job(jenkins, job_name)

    view_name = random_string()
    assert view_name not in jenkins.views
    new_view = jenkins.views.create(view_name)
    assert view_name in jenkins.views
    assert isinstance(new_view, View)

    assert job_name not in new_view
    assert new_view.add_job(job_name)
    assert job_name in new_view
    assert isinstance(new_view[job_name], Job) is True

    assert new_view.remove_job(job_name)
    assert job_name not in new_view
    assert new_view.remove_job(job_name) is False

    del jenkins.views[view_name]



import random
import string


def random_string(length=10):
    return "".join(
        random.choice(string.ascii_lowercase) for i in range(length)
    )


if __name__ == "__main__":
    print(random_string())



import mock

# To run unittests on python 2.6 please use unittest2 library
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from jenkinsapi import config
from jenkinsapi.job import Job
from jenkinsapi.jenkinsbase import JenkinsBase


class TestJobGetAllBuilds(unittest.TestCase):
    # this job has builds
    JOB1_DATA = {
        "actions": [],
        "description": "test job",
        "displayName": "foo",
        "displayNameOrNull": None,
        "name": "foo",
        "url": "http://halob:8080/job/foo/",
        "buildable": True,
        # do as if build 1 & 2 are not returned by jenkins
        "builds": [{"number": 3, "url": "http://halob:8080/job/foo/3/"}],
        "color": "blue",
        "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
        "healthReport": [
            {
                "description": "Build stability: No recent builds failed.",
                "iconUrl": "health-80plus.png",
                "score": 100,
            }
        ],
        "inQueue": False,
        "keepDependencies": False,
        # build running
        "lastBuild": {"number": 4, "url": "http://halob:8080/job/foo/4/"},
        "lastCompletedBuild": {
            "number": 3,
            "url": "http://halob:8080/job/foo/3/",
        },
        "lastFailedBuild": None,
        "lastStableBuild": {
            "number": 3,
            "url": "http://halob:8080/job/foo/3/",
        },
        "lastSuccessfulBuild": {
            "number": 3,
            "url": "http://halob:8080/job/foo/3/",
        },
        "lastUnstableBuild": None,
        "lastUnsuccessfulBuild": None,
        "nextBuildNumber": 4,
        "property": [],
        "queueItem": None,
        "concurrentBuild": False,
        "downstreamProjects": [],
        "scm": {},
        "upstreamProjects": [],
    }
    JOB1_ALL_BUILDS_DATA = {
        "allBuilds": [
            {"number": 3, "url": "http://halob:8080/job/foo/3/"},
            {"number": 2, "url": "http://halob:8080/job/foo/2/"},
            {"number": 1, "url": "http://halob:8080/job/foo/1/"},
        ],
    }
    JOB1_API_URL = "http://halob:8080/job/foo/%s" % config.JENKINS_API

    JOB2_DATA = {
        "actions": [],
        "buildable": True,
        "builds": [],
        "color": "notbuilt",
        "concurrentBuild": False,
        "description": "",
        "displayName": "look_ma_no_builds",
        "displayNameOrNull": None,
        "downstreamProjects": [],
        "firstBuild": None,
        "healthReport": [],
        "inQueue": False,
        "keepDependencies": False,
        "lastBuild": None,
        "lastCompletedBuild": None,
        "lastFailedBuild": None,
        "lastStableBuild": None,
        "lastSuccessfulBuild": None,
        "lastUnstableBuild": None,
        "lastUnsuccessfulBuild": None,
        "name": "look_ma_no_builds",
        "nextBuildNumber": 1,
        "property": [{}],
        "queueItem": None,
        "scm": {},
        "upstreamProjects": [],
        "url": "http://halob:8080/job/look_ma_no_builds/",
    }
    JOB2_API_URL = (
        "http://halob:8080/job/look_ma_no_builds/%s" % config.JENKINS_API
    )

    # Full list available immediatly
    JOB3_DATA = {
        "actions": [],
        "description": "test job",
        "displayName": "fullfoo",
        "displayNameOrNull": None,
        "name": "fullfoo",
        "url": "http://halob:8080/job/fullfoo/",
        "buildable": True,
        # all builds have been returned by Jenkins
        "builds": [
            {"number": 3, "url": "http://halob:8080/job/fullfoo/3/"},
            {"number": 2, "url": "http://halob:8080/job/fullfoo/2/"},
            {"number": 1, "url": "http://halob:8080/job/fullfoo/1/"},
        ],
        "color": "blue",
        "firstBuild": {"number": 1, "url": "http://halob:8080/job/fullfoo/1/"},
        "healthReport": [
            {
                "description": "Build stability: No recent builds failed.",
                "iconUrl": "health-80plus.png",
                "score": 100,
            }
        ],
        "inQueue": False,
        "keepDependencies": False,
        # build running
        "lastBuild": {"number": 4, "url": "http://halob:8080/job/fullfoo/4/"},
        "lastCompletedBuild": {
            "number": 3,
            "url": "http://halob:8080/job/fullfoo/3/",
        },
        "lastFailedBuild": None,
        "lastStableBuild": {
            "number": 3,
            "url": "http://halob:8080/job/fullfoo/3/",
        },
        "lastSuccessfulBuild": {
            "number": 3,
            "url": "http://halob:8080/job/fullfoo/3/",
        },
        "lastUnstableBuild": None,
        "lastUnsuccessfulBuild": None,
        "nextBuildNumber": 4,
        "property": [],
        "queueItem": None,
        "concurrentBuild": False,
        "downstreamProjects": [],
        "scm": {},
        "upstreamProjects": [],
    }
    JOB3_ALL_BUILDS_DATA = {
        "allBuilds": [
            {"number": 3, "url": "http://halob:8080/job/fullfoo/3/"},
            {"number": 2, "url": "http://halob:8080/job/fullfoo/2/"},
            {"number": 1, "url": "http://halob:8080/job/fullfoo/1/"},
        ],
    }
    JOB3_API_URL = "http://halob:8080/job/fullfoo/%s" % config.JENKINS_API

    URL_DATA = {
        JOB1_API_URL: JOB1_DATA,
        (JOB1_API_URL, "allBuilds[number,url]"): JOB1_ALL_BUILDS_DATA,
        JOB2_API_URL: JOB2_DATA,
        JOB3_API_URL: JOB3_DATA,
        # this one below should never be used
        (JOB3_API_URL, "allBuilds[number,url]"): JOB3_ALL_BUILDS_DATA,
    }

    def fakeGetData(self, url, params=None, tree=None):
        TestJobGetAllBuilds.__get_data_call_count += 1
        if params is None:
            try:
                return dict(TestJobGetAllBuilds.URL_DATA[url])
            except KeyError:
                raise Exception("Missing data for url: %s" % url)
        else:
            try:
                return dict(TestJobGetAllBuilds.URL_DATA[(url, str(params))])
            except KeyError:
                raise Exception(
                    "Missing data for url: %s with parameters %s"
                    % (url, repr(params))
                )

    def fakeGetDataTree(self, url, **args):
        TestJobGetAllBuilds.__get_data_call_count += 1
        try:
            if args["tree"]:
                if "builds" in args["tree"]:
                    return {
                        "builds": TestJobGetAllBuilds.URL_DATA[url]["builds"]
                    }
                elif "allBuilds" in args["tree"]:
                    return TestJobGetAllBuilds.URL_DATA[(url, args["tree"])]
                elif "lastBuild" in args["tree"]:
                    return {
                        "lastBuild": TestJobGetAllBuilds.URL_DATA[url][
                            "lastBuild"
                        ]
                    }
            else:
                return dict(TestJobGetAllBuilds.URL_DATA[url])
        except KeyError:
            raise Exception("Missing data for %s" % url)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def setUp(self):
        TestJobGetAllBuilds.__get_data_call_count = 0
        self.J = mock.MagicMock()  # Jenkins object
        self.j = Job("http://halob:8080/job/foo/", "foo", self.J)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_get_build_dict(self):
        # The job data contains only one build, so we expect that the
        # remaining jobs will be fetched automatically
        ret = self.j.get_build_dict()
        self.assertTrue(isinstance(ret, dict))
        self.assertEqual(len(ret), 4)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_incomplete_builds_list_will_call_jenkins_twice(self):
        # The job data contains only one build, so we expect that the
        # remaining jobs will be fetched automatically, and to have two calls
        # to the Jenkins API
        TestJobGetAllBuilds.__get_data_call_count = 0
        self.J.lazy = False
        self.j = Job("http://halob:8080/job/foo/", "foo", self.J)
        self.assertEqual(TestJobGetAllBuilds.__get_data_call_count, 2)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_lazy_builds_list_will_not_call_jenkins_twice(self):
        # The job data contains only one build, so we expect that the
        # remaining jobs will be fetched automatically, and to have two calls
        # to the Jenkins API
        TestJobGetAllBuilds.__get_data_call_count = 0
        self.J.lazy = True
        self.j = Job("http://halob:8080/job/foo/", "foo", self.J)
        self.assertEqual(TestJobGetAllBuilds.__get_data_call_count, 1)
        self.J.lazy = False

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_complete_builds_list_will_call_jenkins_once(self):
        # The job data contains all builds, so we will not gather remaining
        # builds
        TestJobGetAllBuilds.__get_data_call_count = 0
        self.j = Job("http://halob:8080/job/fullfoo/", "fullfoo", self.J)
        self.assertEqual(TestJobGetAllBuilds.__get_data_call_count, 1)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_nobuilds_get_build_dict(self):
        j = Job(
            "http://halob:8080/job/look_ma_no_builds/",
            "look_ma_no_builds",
            self.J,
        )

        ret = j.get_build_dict()
        self.assertTrue(isinstance(ret, dict))
        self.assertEqual(len(ret), 0)

    @mock.patch.object(JenkinsBase, "get_data", fakeGetDataTree)
    def test_get_build_ids(self):
        # The job data contains only one build, so we expect that the
        # remaining jobs will be fetched automatically
        ret = list(self.j.get_build_ids())
        self.assertTrue(isinstance(ret, list))
        self.assertEqual(len(ret), 4)


if __name__ == "__main__":
    unittest.main()



"""
Extract version information from the latest build.
"""

from jenkinsapi.jenkins import Jenkins


job_name = "foo"
jenkins = Jenkins("http://localhost:8080")
job = jenkins[job_name]
lgb = job.get_last_good_build()
print(lgb.get_revision())



"""
System tests for authentication functionality
"""

import pytest
from jenkinsapi.utils.requester import Requester
from requests import HTTPError as REQHTTPError
from jenkinsapi.jenkins import Jenkins


def test_normal_authentication(jenkins_admin_admin):
    # No problem with the righ user/pass
    jenkins_user = Jenkins(
        jenkins_admin_admin.baseurl,
        jenkins_admin_admin.username,
        jenkins_admin_admin.password,
    )

    assert jenkins_user is not None

    # We cannot connect if no user/pass
    with pytest.raises(REQHTTPError) as http_excep:
        Jenkins(jenkins_admin_admin.baseurl)

    assert Requester.AUTH_COOKIE is None
    assert http_excep.value.response.status_code == 403


#    def test_auth_cookie(jenkins_admin_admin):
#        initial_cookie_value = None
#        final_cookie_value = "JSESSIONID"
#        assert initial_cookie_value == Requester.AUTH_COOKIE
#
#        jenkins_admin_admin.use_auth_cookie()
#
#        result = Requester.AUTH_COOKIE
#        assert result is not None
#        assert final_cookie_value in result
#
#
#    def test_wrongauth_cookie(jenkins_admin_admin):
#        initial_cookie_value = None
#        assert initial_cookie_value == Requester.AUTH_COOKIE
#
#        jenkins_admin_admin.username = "fakeuser"
#        jenkins_admin_admin.password = "fakepass"
#
#        with pytest.raises(HTTPError) as http_excep:
#            jenkins_admin_admin.use_auth_cookie()
#
#        assert Requester.AUTH_COOKIE is None
#        assert http_excep.value.code == 401
#
#
#    def test_verify_cookie_isworking(jenkins_admin_admin):
#        initial_cookie_value = None
#        final_cookie_value = "JSESSIONID"
#        assert initial_cookie_value == Requester.AUTH_COOKIE
#
#        # Remove requester user/pass
#        jenkins_admin_admin.requester.username = None
#        jenkins_admin_admin.requester.password = None
#
#        # Verify that we cannot connect
#        with pytest.raises(REQHTTPError) as http_excep:
#            jenkins_admin_admin.poll()
#
#        assert Requester.AUTH_COOKIE is None
#        assert http_excep.value.response.status_code == 403
#
#        # Retrieve the auth cookie, we can because we
#        # have right values for jenkins_admin_admin.username
#        # and jenkins_admin_admin.password
#        jenkins_admin_admin.use_auth_cookie()
#
#        result = Requester.AUTH_COOKIE
#        assert result is not None
#        assert final_cookie_value in result
#
#        # Verify that we can connect even with no requester user/pass
#        # If we have the cookie the requester user/pass is not needed
#        jenkins_admin_admin.poll()



"""
System tests for `jenkinsapi.jenkins` module.
"""

import pytest
from jenkinsapi.job import Job
from jenkinsapi.jobs import Jobs
from jenkinsapi.build import Build
from jenkinsapi.queue import QueueItem
from jenkinsapi_tests.systests.job_configs import EMPTY_JOB
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi.custom_exceptions import UnknownJob


def job_present(jenkins, name):
    jenkins.poll()
    assert name in jenkins, "Job %r is absent in jenkins." % name
    assert isinstance(jenkins.get_job(name), Job) is True
    assert isinstance(jenkins[name], Job) is True


def job_absent(jenkins, name):
    jenkins.poll()
    assert name not in jenkins, "Job %r is present in jenkins." % name


def test_create_job(jenkins):
    job_name = "create_%s" % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)


def test_create_job_with_plus(jenkins):
    job_name = "create+%s" % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    job = jenkins[job_name]
    assert job_name in job.url


def test_create_dup_job(jenkins):
    job_name = "create_%s" % random_string()
    old_job = jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    new_job = jenkins.create_job(job_name, EMPTY_JOB)
    assert new_job == old_job


def test_get_jobs_info(jenkins):
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)

    jobs_info = list(jenkins.get_jobs_info())
    assert len(jobs_info) == 1
    for url, name in jobs_info:
        assert url == job.url
        assert name == job.name


def test_create_job_through_jobs_dict(jenkins):
    job_name = "create_%s" % random_string()
    jenkins.jobs[job_name] = EMPTY_JOB
    job_present(jenkins, job_name)


def test_enable_disable_job(jenkins):
    job_name = "create_%s" % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)

    j = jenkins[job_name]
    j.invoke(block=True)  # run this at least once
    # Ensure job begins as enabled
    assert j.is_enabled() is True, "An enabled job is reporting incorrectly"

    j.disable()
    assert j.is_enabled() is False, "A disabled job is reporting incorrectly"

    j.enable()
    assert j.is_enabled() is True, "An enabled job is reporting incorrectly"


def test_get_job_and_update_config(jenkins):
    job_name = "config_%s" % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    config = jenkins[job_name].get_config()
    assert config.strip() == EMPTY_JOB.strip()
    jenkins[job_name].update_config(EMPTY_JOB)


def test_invoke_job(jenkins):
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)
    job.invoke(block=True)
    assert isinstance(job.get_build(1), Build)


def test_invocation_object(jenkins):
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)
    ii = job.invoke()
    assert isinstance(ii, QueueItem) is True


def test_get_jobs_list(jenkins):
    job1_name = "first_%s" % random_string()
    job2_name = "second_%s" % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.create_job(job2_name, EMPTY_JOB)
    assert len(jenkins.jobs) >= 2
    job_list = jenkins.get_jobs_list()
    assert [job1_name, job2_name] == job_list


def test_get_job(jenkins):
    job1_name = "first_%s" % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    job = jenkins[job1_name]
    assert isinstance(job, Job) is True
    assert job.name == job1_name


def test_get_jobs(jenkins):
    job1_name = "first_%s" % random_string()
    job2_name = "second_%s" % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.create_job(job2_name, EMPTY_JOB)
    jobs = jenkins.jobs
    assert isinstance(jobs, Jobs) is True
    assert len(jobs) >= 2
    for job_name, job in jobs.iteritems():
        assert isinstance(job_name, str) is True
        assert isinstance(job, Job) is True


def test_get_job_that_does_not_exist(jenkins):
    with pytest.raises(UnknownJob):
        jenkins["doesnot_exist"]


def test_has_job(jenkins):
    job1_name = "first_%s" % random_string()
    jenkins.create_job(job1_name, EMPTY_JOB)
    assert jenkins.has_job(job1_name) is True
    assert job1_name in jenkins


def test_has_no_job(jenkins):
    assert jenkins.has_job("doesnt_exist") is False
    assert "doesnt_exist" not in jenkins


def test_delete_job(jenkins):
    job1_name = "delete_me_%s" % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.delete_job(job1_name)
    job_absent(jenkins, job1_name)


def test_rename_job(jenkins):
    job1_name = "A__%s" % random_string()
    job2_name = "B__%s" % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.rename_job(job1_name, job2_name)
    job_absent(jenkins, job1_name)
    job_present(jenkins, job2_name)


def test_copy_job(jenkins):
    template_job_name = "TPL%s" % random_string()
    copied_job_name = "CPY%s" % random_string()

    jenkins.create_job(template_job_name, EMPTY_JOB)
    j = jenkins.copy_job(template_job_name, copied_job_name)
    job_present(jenkins, template_job_name)
    job_present(jenkins, copied_job_name)
    assert isinstance(j, Job) is True
    assert j.name == copied_job_name


def test_get_master_data(jenkins):
    master_data = jenkins.get_master_data()
    assert master_data["totalExecutors"] == 2


def test_run_groovy_script(jenkins):
    expected_result = "Hello world!"
    result = jenkins.run_groovy_script('print "%s"' % expected_result)
    assert result.strip() == "Hello world!"



import pytest
from collections import namedtuple

import jenkinsapi
from jenkinsapi.plugins import Plugins
from jenkinsapi.utils.requester import Requester
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import JenkinsAPIException

DATA = {}
TWO_JOBS_DATA = {
    "jobs": [
        {
            "name": "job_one",
            "url": "http://localhost:8080/job/job_one",
            "color": "blue",
        },
        {
            "name": "job_two",
            "url": "http://localhost:8080/job/job_two",
            "color": "blue",
        },
    ]
}
MULTIBRANCH_JOBS_DATA = {
    "jobs": [
        {
            "name": "multibranch-repo/master",
            "url": "http://localhost:8080/job/multibranch-repo/job/master",
            "color": "blue",
        },
        {
            "name": "multibranch-repo/develop",
            "url": "http://localhost:8080/job/multibranch-repo/job/develop",
            "color": "blue",
        },
    ]
}
SCAN_MULTIBRANCH_PIPELINE_LOG = """
Started by timer
[Fri Jul 05 06:46:00 CEST 2019] Starting branch indexing...
Connecting to https://stash.macq.eu using Jenkins/****** (jenkins-ldap)
Repository type: Git
Looking up internal/base for branches
Checking branch master from internal/base
      'Jenkinsfile' found
    Met criteria
No changes detected: master (still at 26d4d8a673f57a957fd5a23f5adfe0be02089294)

  1 branches were processed
Looking up internal/base for pull requests

  0 pull requests were processed
[Fri Jul 05 06:46:01 CEST 2019] Finished branch indexing. Indexing took 1.1 sec
Finished: SUCCESS
"""


@pytest.fixture(scope="function")
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)

    return Jenkins(
        "http://localhost:8080", username="foouser", password="foopassword"
    )


def test__clone(jenkins):
    cloned = jenkins._clone()
    assert id(cloned) != id(jenkins)
    assert cloned == jenkins


def test_stored_passwords(jenkins):
    assert jenkins.requester.password == "foopassword"
    assert jenkins.requester.username == "foouser"


def test_reload(monkeypatch):
    class FakeResponse(object):
        status_code = 200
        text = "{}"

    def fake_get_url(
        url,  # pylint: disable=unused-argument
        params=None,  # pylint: disable=unused-argument
        headers=None,  # pylint: disable=unused-argument
        allow_redirects=True,  # pylint: disable=unused-argument
        stream=False,
    ):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(Requester, "get_url", fake_get_url)
    mock_requester = Requester(username="foouser", password="foopassword")
    jenkins = Jenkins(
        "http://localhost:8080/",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )
    jenkins.poll()


def test_get_jobs_list(monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Job, "_poll", fake_job_poll)

    jenkins = Jenkins(
        "http://localhost:8080/", username="foouser", password="foopassword"
    )
    for idx, job_name in enumerate(jenkins.get_jobs_list()):
        assert job_name == TWO_JOBS_DATA["jobs"][idx]["name"]

    for idx, job_name in enumerate(jenkins.jobs.keys()):
        assert job_name == TWO_JOBS_DATA["jobs"][idx]["name"]


def test_create_new_job_fail(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Job, "_poll", fake_job_poll)

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=""
    )

    jenkins = Jenkins(
        "http://localhost:8080/",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )

    with pytest.raises(JenkinsAPIException) as ar:
        jenkins.create_job("job_new", None)

    assert "Job XML config cannot be empty" in str(ar.value)


def test_create_multibranch_pipeline_job(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        # return multibranch jobs and other jobs.
        # create_multibranch_pipeline_job is supposed to filter out
        # the MULTIBRANCH jobs
        return {"jobs": TWO_JOBS_DATA["jobs"] + MULTIBRANCH_JOBS_DATA["jobs"]}

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Job, "_poll", fake_job_poll)

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=""
    )
    mock_requester.post_and_confirm_status = mocker.MagicMock(return_value="")
    get_response = namedtuple("get_response", "text")
    mock_requester.get_url = mocker.MagicMock(
        return_value=get_response(text=SCAN_MULTIBRANCH_PIPELINE_LOG)
    )
    jenkins = Jenkins(
        "http://localhost:8080/",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )

    jobs = jenkins.create_multibranch_pipeline_job(
        "multibranch-repo", "multibranch-xml-content"
    )

    for idx, job_instance in enumerate(jobs):
        assert job_instance.name == MULTIBRANCH_JOBS_DATA["jobs"][idx]["name"]

    # make sure we didn't get more jobs.
    assert len(MULTIBRANCH_JOBS_DATA["jobs"]) == len(jobs)


def test_get_jenkins_obj_from_url(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Job, "_poll", fake_job_poll)

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=""
    )

    jenkins = Jenkins(
        "http://localhost:8080/",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )

    new_jenkins = jenkins.get_jenkins_obj_from_url("http://localhost:8080/")
    assert new_jenkins == jenkins

    new_jenkins = jenkins.get_jenkins_obj_from_url("http://localhost:8080/foo")
    assert new_jenkins != jenkins


def test_get_jenkins_obj(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, "_poll", fake_jenkins_poll)
    monkeypatch.setattr(Job, "_poll", fake_job_poll)

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=""
    )

    jenkins = Jenkins(
        "http://localhost:8080/",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )

    new_jenkins = jenkins.get_jenkins_obj()
    assert new_jenkins == jenkins


def test_get_version(monkeypatch):
    class MockResponse(object):
        def __init__(self):
            self.headers = {}
            self.headers["X-Jenkins"] = "1.542"

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)

    def fake_get(cls, *arga, **kwargs):  # pylint: disable=unused-argument
        return MockResponse()

    monkeypatch.setattr(Requester, "get_and_confirm_status", fake_get)

    jenkins = Jenkins(
        "http://foobar:8080/", username="foouser", password="foopassword"
    )
    assert jenkins.version == "1.542"


def test_get_version_nonexistent(mocker):
    class MockResponse(object):
        status_code = 200
        headers = {}
        text = "{}"

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.get_url = mocker.MagicMock(return_value=MockResponse())
    jenkins = Jenkins(
        "http://localhost:8080",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )
    assert jenkins.version == "0.0"


def test_get_master_data(mocker):
    class MockResponse(object):
        status_code = 200
        headers = {}
        text = "{}"

    mock_requester = Requester(username="foouser", password="foopassword")
    mock_requester.get_url = mocker.MagicMock(return_value=MockResponse())
    jenkins = Jenkins(
        "http://localhost:808",
        username="foouser",
        password="foopassword",
        requester=mock_requester,
    )
    jenkins.get_data = mocker.MagicMock(
        return_value={"busyExecutors": 59, "totalExecutors": 75}
    )

    data = jenkins.get_master_data()
    assert data["busyExecutors"] == 59
    assert data["totalExecutors"] == 75


def test_get_create_url(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)
    # Jenkins URL w/o slash
    jenkins = Jenkins(
        "http://localhost:8080", username="foouser", password="foopassword"
    )
    assert jenkins.get_create_url() == "http://localhost:8080/createItem"
    # Jenkins URL w/ slash
    jenkins = Jenkins(
        "http://localhost:8080/", username="foouser", password="foopassword"
    )
    assert jenkins.get_create_url() == "http://localhost:8080/createItem"


def test_has_plugin(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)

    def fake_plugin_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {
            "plugins": [
                {
                    "deleted": False,
                    "hasUpdate": True,
                    "downgradable": False,
                    "dependencies": [{}, {}, {}, {}],
                    "longName": "Jenkins Subversion Plug-in",
                    "active": True,
                    "shortName": "subversion",
                    "backupVersion": None,
                    "url": "http://wiki.jenkins-ci.org/"
                    "display/JENKINS/Subversion+Plugin",
                    "enabled": True,
                    "pinned": False,
                    "version": "1.45",
                    "supportsDynamicLoad": "MAYBE",
                    "bundled": True,
                }
            ]
        }

    monkeypatch.setattr(Plugins, "_poll", fake_plugin_poll)

    jenkins = Jenkins(
        "http://localhost:8080/", username="foouser", password="foopassword"
    )
    assert jenkins.has_plugin("subversion") is True


def test_get_use_auth_cookie(mocker, monkeypatch):
    COOKIE_VALUE = "FAKE_COOKIE"

    def fake_opener(redirect_handler):  # pylint: disable=unused-argument
        mock_response = mocker.MagicMock()
        mock_response.cookie = COOKIE_VALUE

        mock_opener = mocker.MagicMock()
        mock_opener.open.return_value = mock_response
        return mock_opener

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)
    monkeypatch.setattr(Requester, "AUTH_COOKIE", None)
    monkeypatch.setattr(jenkinsapi.jenkins, "build_opener", fake_opener)

    jenkins = Jenkins(
        "http://localhost:8080", username="foouser", password="foopassword"
    )

    jenkins.use_auth_cookie()
    assert Requester.AUTH_COOKIE == COOKIE_VALUE
